import logging
import sys

from pykka import registry, actor

from mopidy import listeners, settings
from mopidy.frontends.mpd import dispatcher, protocol
from mopidy.utils import network, process, log

logger = logging.getLogger('mopidy.frontends.mpd')

class MpdFrontend(actor.ThreadingActor, listeners.BackendListener):
    """
    The MPD frontend.

    **Dependencies:**

    - None

    **Settings:**

    - :attr:`mopidy.settings.MPD_SERVER_HOSTNAME`
    - :attr:`mopidy.settings.MPD_SERVER_PORT`
    - :attr:`mopidy.settings.MPD_SERVER_PASSWORD`
    """

    def __init__(self):
        super(MpdFrontend, self).__init__()
        hostname = network.format_hostname(settings.MPD_SERVER_HOSTNAME)
        port = settings.MPD_SERVER_PORT

        try:
            network.Server(hostname, port, protocol=MpdSession,
                max_connections=settings.MPD_SERVER_MAX_CONNECTIONS)
        except IOError, e:
            logger.error(u'MPD server startup failed: %s', e)
            sys.exit(1)

        logger.info(u'MPD server running at [%s]:%s', hostname, port)

    def on_stop(self):
        process.stop_actors_by_class(MpdSession)

    def send_idle(self, subsystem):
        # FIXME this should be updated once pykka supports non-blocking calls
        # on proxies or some similar solution
        registry.ActorRegistry.broadcast({
            'command': 'pykka_call',
            'attr_path': ('on_idle',),
            'args': [subsystem],
            'kwargs': {},
        }, target_class=MpdSession)

    def playback_state_changed(self):
        self.send_idle('player')

    def playlist_changed(self):
        self.send_idle('playlist')

    def options_changed(self):
        self.send_idle('options')

    def volume_changed(self):
        self.send_idle('mixer')


class MpdSession(network.LineProtocol):
    """
    The MPD client session. Keeps track of a single client session. Any
    requests from the client is passed on to the MPD request dispatcher.
    """

    terminator = protocol.LINE_TERMINATOR
    encoding = protocol.ENCODING
    delimeter = r'\r?\n'

    def __init__(self, connection):
        super(MpdSession, self).__init__(connection)
        self.dispatcher = dispatcher.MpdDispatcher(self)

    def on_start(self):
        logger.info(u'New MPD connection from [%s]:%s', self.host, self.port)
        self.send_lines([u'OK MPD %s' % protocol.VERSION])

    def on_line_received(self, line):
        logger.debug(u'Request from [%s]:%s to %s: %s', self.host, self.port,
            self.actor_urn, line)

        response = self.dispatcher.handle_request(line)
        if not response:
            return

        logger.debug(u'Response to [%s]:%s from %s: %s', self.host, self.port,
            self.actor_urn, log.indent(self.terminator.join(response)))

        self.send_lines(response)

    def on_idle(self, subsystem):
        self.dispatcher.handle_idle(subsystem)

    def decode(self, line):
        try:
            return super(MpdSession, self).decode(line.decode('string_escape'))
        except ValueError:
            logger.warning(u'Stopping actor due to unescaping error, data '
                'supplied by client was not valid.')
            self.stop()

    def close(self):
        self.stop()



import sys

import mock

from mopidy import OptionalDependencyError
from mopidy.models import Track

try:
    from mopidy.frontends.mpris import MprisFrontend, objects
except OptionalDependencyError:
    pass

from tests import unittest


@unittest.skipUnless(sys.platform.startswith('linux'), 'requires Linux')
class BackendEventsTest(unittest.TestCase):
    def setUp(self):
        self.mpris_frontend = MprisFrontend() # As a plain class, not an actor
        self.mpris_object = mock.Mock(spec=objects.MprisObject)
        self.mpris_frontend.mpris_object = self.mpris_object

    def test_track_playback_paused_event_changes_playback_status(self):
        self.mpris_object.Get.return_value = 'Paused'
        self.mpris_frontend.track_playback_paused(Track(), 0)
        self.assertListEqual(self.mpris_object.Get.call_args_list, [
            ((objects.PLAYER_IFACE, 'PlaybackStatus'), {}),
        ])
        self.mpris_object.PropertiesChanged.assert_called_with(
            objects.PLAYER_IFACE, {'PlaybackStatus': 'Paused'}, [])

    def test_track_playback_resumed_event_changes_playback_status(self):
        self.mpris_object.Get.return_value = 'Playing'
        self.mpris_frontend.track_playback_resumed(Track(), 0)
        self.assertListEqual(self.mpris_object.Get.call_args_list, [
            ((objects.PLAYER_IFACE, 'PlaybackStatus'), {}),
        ])
        self.mpris_object.PropertiesChanged.assert_called_with(
            objects.PLAYER_IFACE, {'PlaybackStatus': 'Playing'}, [])

    def test_track_playback_started_event_changes_playback_status_and_metadata(self):
        self.mpris_object.Get.return_value = '...'
        self.mpris_frontend.track_playback_started(Track())
        self.assertListEqual(self.mpris_object.Get.call_args_list, [
            ((objects.PLAYER_IFACE, 'PlaybackStatus'), {}),
            ((objects.PLAYER_IFACE, 'Metadata'), {}),
        ])
        self.mpris_object.PropertiesChanged.assert_called_with(
            objects.PLAYER_IFACE,
            {'Metadata': '...', 'PlaybackStatus': '...'}, [])

    def test_track_playback_ended_event_changes_playback_status_and_metadata(self):
        self.mpris_object.Get.return_value = '...'
        self.mpris_frontend.track_playback_ended(Track(), 0)
        self.assertListEqual(self.mpris_object.Get.call_args_list, [
            ((objects.PLAYER_IFACE, 'PlaybackStatus'), {}),
            ((objects.PLAYER_IFACE, 'Metadata'), {}),
        ])
        self.mpris_object.PropertiesChanged.assert_called_with(
            objects.PLAYER_IFACE,
            {'Metadata': '...', 'PlaybackStatus': '...'}, [])

    def test_volume_changed_event_changes_volume(self):
        self.mpris_object.Get.return_value = 1.0
        self.mpris_frontend.volume_changed()
        self.assertListEqual(self.mpris_object.Get.call_args_list, [
            ((objects.PLAYER_IFACE, 'Volume'), {}),
        ])
        self.mpris_object.PropertiesChanged.assert_called_with(
            objects.PLAYER_IFACE, {'Volume': 1.0}, [])

    def test_seeked_event_causes_mpris_seeked_event(self):
        self.mpris_object.Get.return_value = 31000000
        self.mpris_frontend.seeked()
        self.assertListEqual(self.mpris_object.Get.call_args_list, [
            ((objects.PLAYER_IFACE, 'Position'), {}),
        ])
        self.mpris_object.Seeked.assert_called_with(31000000)



import datetime

from mopidy.models import Track, Playlist

from tests.frontends.mpd import protocol


class StoredPlaylistsHandlerTest(protocol.BaseTestCase):
    def test_listplaylist(self):
        self.backend.stored_playlists.playlists = [
            Playlist(name='name', tracks=[Track(uri='file:///dev/urandom')])]

        self.sendRequest(u'listplaylist "name"')
        self.assertInResponse(u'file: file:///dev/urandom')
        self.assertInResponse(u'OK')

    def test_listplaylist_fails_if_no_playlist_is_found(self):
        self.sendRequest(u'listplaylist "name"')
        self.assertEqualResponse(u'ACK [50@0] {listplaylist} No such playlist')

    def test_listplaylistinfo(self):
        self.backend.stored_playlists.playlists = [
            Playlist(name='name', tracks=[Track(uri='file:///dev/urandom')])]

        self.sendRequest(u'listplaylistinfo "name"')
        self.assertInResponse(u'file: file:///dev/urandom')
        self.assertInResponse(u'Track: 0')
        self.assertNotInResponse(u'Pos: 0')
        self.assertInResponse(u'OK')

    def test_listplaylistinfo_fails_if_no_playlist_is_found(self):
        self.sendRequest(u'listplaylistinfo "name"')
        self.assertEqualResponse(
            u'ACK [50@0] {listplaylistinfo} No such playlist')

    def test_listplaylists(self):
        last_modified = datetime.datetime(2001, 3, 17, 13, 41, 17, 12345)
        self.backend.stored_playlists.playlists = [Playlist(name='a',
            last_modified=last_modified)]

        self.sendRequest(u'listplaylists')
        self.assertInResponse(u'playlist: a')
        # Date without microseconds and with time zone information
        self.assertInResponse(u'Last-Modified: 2001-03-17T13:41:17Z')
        self.assertInResponse(u'OK')

    def test_load_known_playlist_appends_to_current_playlist(self):
        self.backend.current_playlist.append([Track(uri='a'), Track(uri='b')])
        self.assertEqual(len(self.backend.current_playlist.tracks.get()), 2)
        self.backend.stored_playlists.playlists = [Playlist(name='A-list',
            tracks=[Track(uri='c'), Track(uri='d'), Track(uri='e')])]

        self.sendRequest(u'load "A-list"')
        tracks = self.backend.current_playlist.tracks.get()
        self.assertEqual(5, len(tracks))
        self.assertEqual('a', tracks[0].uri)
        self.assertEqual('b', tracks[1].uri)
        self.assertEqual('c', tracks[2].uri)
        self.assertEqual('d', tracks[3].uri)
        self.assertEqual('e', tracks[4].uri)
        self.assertInResponse(u'OK')

    def test_load_unknown_playlist_acks(self):
        self.sendRequest(u'load "unknown playlist"')
        self.assertEqual(0, len(self.backend.current_playlist.tracks.get()))
        self.assertEqualResponse(u'ACK [50@0] {load} No such playlist')

    def test_playlistadd(self):
        self.sendRequest(u'playlistadd "name" "file:///dev/urandom"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_playlistclear(self):
        self.sendRequest(u'playlistclear "name"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_playlistdelete(self):
        self.sendRequest(u'playlistdelete "name" "5"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_playlistmove(self):
        self.sendRequest(u'playlistmove "name" "5" "10"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_rename(self):
        self.sendRequest(u'rename "old_name" "new_name"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_rm(self):
        self.sendRequest(u'rm "name"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_save(self):
        self.sendRequest(u'save "name"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')



import gobject
gobject.threads_init()

import pygst
pygst.require('0.10')
import gst

import datetime

from mopidy.utils.path import path_to_uri, find_files
from mopidy.models import Track, Artist, Album

def translator(data):
    albumartist_kwargs = {}
    album_kwargs = {}
    artist_kwargs = {}
    track_kwargs = {}

    def _retrieve(source_key, target_key, target):
        if source_key in data:
            target[target_key] = data[source_key]

    _retrieve(gst.TAG_ALBUM, 'name', album_kwargs)
    _retrieve(gst.TAG_TRACK_COUNT, 'num_tracks', album_kwargs)
    _retrieve(gst.TAG_ARTIST, 'name', artist_kwargs)

    if gst.TAG_DATE in data and data[gst.TAG_DATE]:
        date = data[gst.TAG_DATE]
        date = datetime.date(date.year, date.month, date.day)
        track_kwargs['date'] = date

    _retrieve(gst.TAG_TITLE, 'name', track_kwargs)
    _retrieve(gst.TAG_TRACK_NUMBER, 'track_no', track_kwargs)

    # Following keys don't seem to have TAG_* constant.
    _retrieve('album-artist', 'name', albumartist_kwargs)
    _retrieve('musicbrainz-trackid', 'musicbrainz_id', track_kwargs)
    _retrieve('musicbrainz-artistid', 'musicbrainz_id', artist_kwargs)
    _retrieve('musicbrainz-albumid', 'musicbrainz_id', album_kwargs)
    _retrieve('musicbrainz-albumartistid', 'musicbrainz_id', albumartist_kwargs)

    if albumartist_kwargs:
        album_kwargs['artists'] = [Artist(**albumartist_kwargs)]

    track_kwargs['uri'] = data['uri']
    track_kwargs['length'] = data[gst.TAG_DURATION]
    track_kwargs['album'] = Album(**album_kwargs)
    track_kwargs['artists'] = [Artist(**artist_kwargs)]

    return Track(**track_kwargs)


class Scanner(object):
    def __init__(self, folder, data_callback, error_callback=None):
        self.uris = [path_to_uri(f) for f in find_files(folder)]
        self.data_callback = data_callback
        self.error_callback = error_callback
        self.loop = gobject.MainLoop()

        fakesink = gst.element_factory_make('fakesink')

        self.uribin = gst.element_factory_make('uridecodebin')
        self.uribin.set_property('caps', gst.Caps('audio/x-raw-int'))
        self.uribin.connect('pad-added', self.process_new_pad,
            fakesink.get_pad('sink'))

        self.pipe = gst.element_factory_make('pipeline')
        self.pipe.add(self.uribin)
        self.pipe.add(fakesink)

        bus = self.pipe.get_bus()
        bus.add_signal_watch()
        bus.connect('message::tag', self.process_tags)
        bus.connect('message::error', self.process_error)

    def process_new_pad(self, source, pad, target_pad):
        pad.link(target_pad)

    def process_tags(self, bus, message):
        taglist = message.parse_tag()
        data = {
            'uri': unicode(self.uribin.get_property('uri')),
            gst.TAG_DURATION: self.get_duration(),
        }

        for key in taglist.keys():
            # XXX: For some crazy reason some wma files spit out lists here,
            # not sure if this is due to better data in headers or wma being
            # stupid. So ugly hack for now :/
            if type(taglist[key]) is list:
                data[key] = taglist[key][0]
            else:
                data[key] = taglist[key]

        try:
            self.data_callback(data)
            self.next_uri()
        except KeyboardInterrupt:
            self.stop()

    def process_error(self, bus, message):
        if self.error_callback:
            uri = self.uribin.get_property('uri')
            error, debug = message.parse_error()
            self.error_callback(uri, error, debug)
        self.next_uri()

    def get_duration(self):
        self.pipe.get_state() # Block until state change is done.
        try:
            return self.pipe.query_duration(
                gst.FORMAT_TIME, None)[0] // gst.MSECOND
        except gst.QueryError:
            return None

    def next_uri(self):
        if not self.uris:
            return self.stop()

        self.pipe.set_state(gst.STATE_NULL)
        self.uribin.set_property('uri', self.uris.pop())
        self.pipe.set_state(gst.STATE_PAUSED)

    def start(self):
        if not self.uris:
            return
        self.next_uri()
        self.loop.run()

    def stop(self):
        self.pipe.set_state(gst.STATE_NULL)
        self.loop.quit()



import os
import subprocess
import sys

import mopidy

from tests import unittest


class HelpTest(unittest.TestCase):
    def test_help_has_mopidy_options(self):
        mopidy_dir = os.path.dirname(mopidy.__file__)
        args = [sys.executable, mopidy_dir, '--help']
        process = subprocess.Popen(args, stdout=subprocess.PIPE)
        output = process.communicate()[0]
        self.assert_('--version' in output)
        self.assert_('--help' in output)
        self.assert_('--help-gst' in output)
        self.assert_('--interactive' in output)
        self.assert_('--quiet' in output)
        self.assert_('--verbose' in output)
        self.assert_('--save-debug-log' in output)
        self.assert_('--list-settings' in output)

    def test_help_gst_has_gstreamer_options(self):
        mopidy_dir = os.path.dirname(mopidy.__file__)
        args = [sys.executable, mopidy_dir, '--help-gst']
        process = subprocess.Popen(args, stdout=subprocess.PIPE)
        output = process.communicate()[0]
        self.assert_('--gst-version' in output)



import os
import shutil
import tempfile

from mopidy import settings
from mopidy.models import Playlist

from tests import unittest, path_to_data_dir


class StoredPlaylistsControllerTest(object):
    def setUp(self):
        settings.LOCAL_PLAYLIST_PATH = tempfile.mkdtemp()
        settings.LOCAL_TAG_CACHE_FILE = path_to_data_dir('library_tag_cache')
        settings.LOCAL_MUSIC_PATH = path_to_data_dir('')

        self.backend = self.backend_class()
        self.stored  = self.backend.stored_playlists

    def tearDown(self):
        if os.path.exists(settings.LOCAL_PLAYLIST_PATH):
            shutil.rmtree(settings.LOCAL_PLAYLIST_PATH)

        settings.runtime.clear()

    def test_create(self):
        playlist = self.stored.create('test')
        self.assertEqual(playlist.name, 'test')

    def test_create_in_playlists(self):
        playlist = self.stored.create('test')
        self.assert_(self.stored.playlists)
        self.assert_(playlist in self.stored.playlists)

    def test_playlists_empty_to_start_with(self):
        self.assert_(not self.stored.playlists)

    def test_delete_non_existant_playlist(self):
        self.stored.delete(Playlist())

    def test_delete_playlist(self):
        playlist = self.stored.create('test')
        self.stored.delete(playlist)
        self.assert_(not self.stored.playlists)

    def test_get_without_criteria(self):
        test = self.stored.get
        self.assertRaises(LookupError, test)

    def test_get_with_wrong_cirteria(self):
        test = lambda: self.stored.get(name='foo')
        self.assertRaises(LookupError, test)

    def test_get_with_right_criteria(self):
        playlist1 = self.stored.create('test')
        playlist2 = self.stored.get(name='test')
        self.assertEqual(playlist1, playlist2)

    def test_get_by_name_returns_unique_match(self):
        playlist = Playlist(name='b')
        self.stored.playlists = [Playlist(name='a'), playlist]
        self.assertEqual(playlist, self.stored.get(name='b'))

    def test_get_by_name_returns_first_of_multiple_matches(self):
        playlist = Playlist(name='b')
        self.stored.playlists = [
            playlist, Playlist(name='a'), Playlist(name='b')]
        try:
            self.stored.get(name='b')
            self.fail(u'Should raise LookupError if multiple matches')
        except LookupError as e:
            self.assertEqual(u'"name=b" match multiple playlists', e[0])

    def test_get_by_name_raises_keyerror_if_no_match(self):
        self.stored.playlists = [Playlist(name='a'), Playlist(name='b')]
        try:
            self.stored.get(name='c')
            self.fail(u'Should raise LookupError if no match')
        except LookupError as e:
            self.assertEqual(u'"name=c" match no playlists', e[0])

    @unittest.SkipTest
    def test_lookup(self):
        pass

    @unittest.SkipTest
    def test_refresh(self):
        pass

    def test_rename(self):
        playlist = self.stored.create('test')
        self.stored.rename(playlist, 'test2')
        self.stored.get(name='test2')

    def test_rename_unknown_playlist(self):
        self.stored.rename(Playlist(), 'test2')
        test = lambda: self.stored.get(name='test2')
        self.assertRaises(LookupError, test)

    def test_save(self):
        # FIXME should we handle playlists without names?
        playlist = Playlist(name='test')
        self.stored.save(playlist)
        self.assert_(playlist in self.stored.playlists)

    @unittest.SkipTest
    def test_playlist_with_unknown_track(self):
        pass



from mopidy.mixers.denon import DenonMixer
from tests.mixers.base_test import BaseMixerTest

from tests import unittest


class DenonMixerDeviceMock(object):
    def __init__(self):
        self._open = True
        self.ret_val = bytes('MV00\r')

    def write(self, x):
        if x[2] != '?':
            self.ret_val = bytes(x)

    def read(self, x):
        return self.ret_val

    def readline(self):
        return self.ret_val

    def isOpen(self):
        return self._open

    def open(self):
        self._open = True


class DenonMixerTest(BaseMixerTest, unittest.TestCase):
    ACTUAL_MAX = 99
    INITIAL = 1

    mixer_class = DenonMixer

    def setUp(self):
        self.device = DenonMixerDeviceMock()
        self.mixer = DenonMixer(device=self.device)

    def test_reopen_device(self):
        self.device._open = False
        self.mixer.volume = 10
        self.assertTrue(self.device.isOpen())



import logging
import os

logger = logging.getLogger('mopidy.backends.local.translator')

from mopidy.models import Track, Artist, Album
from mopidy.utils.path import path_to_uri

def parse_m3u(file_path):
    """
    Convert M3U file list of uris

    Example M3U data::

        # This is a comment
        Alternative\Band - Song.mp3
        Classical\Other Band - New Song.mp3
        Stuff.mp3
        D:\More Music\Foo.mp3
        http://www.example.com:8000/Listen.pls
        http://www.example.com/~user/Mine.mp3

    - Relative paths of songs should be with respect to location of M3U.
    - Paths are normaly platform specific.
    - Lines starting with # should be ignored.
    - m3u files are latin-1.
    - This function does not bother with Extended M3U directives.
    """

    uris = []
    folder = os.path.dirname(file_path)

    try:
        with open(file_path) as m3u:
            contents = m3u.readlines()
    except IOError, e:
        logger.error('Couldn\'t open m3u: %s', e)
        return uris

    for line in contents:
        line = line.strip().decode('latin1')

        if line.startswith('#'):
            continue

        # FIXME what about other URI types?
        if line.startswith('file://'):
            uris.append(line)
        else:
            path = path_to_uri(folder, line)
            uris.append(path)

    return uris

def parse_mpd_tag_cache(tag_cache, music_dir=''):
    """
    Converts a MPD tag_cache into a lists of tracks, artists and albums.
    """
    tracks = set()

    try:
        with open(tag_cache) as library:
            contents = library.read()
    except IOError, e:
        logger.error('Could not open tag cache: %s', e)
        return tracks

    current = {}
    state = None

    for line in contents.split('\n'):
        if line == 'songList begin':
            state = 'songs'
            continue
        elif line == 'songList end':
            state = None
            continue
        elif not state:
            continue

        key, value = line.split(': ', 1)

        if key == 'key':
            _convert_mpd_data(current, tracks, music_dir)
            current.clear()

        current[key.lower()] = value.decode('utf-8')

    _convert_mpd_data(current, tracks, music_dir)

    return tracks

def _convert_mpd_data(data, tracks, music_dir):
    if not data:
        return

    track_kwargs = {}
    album_kwargs = {}
    artist_kwargs = {}
    albumartist_kwargs = {}

    if 'track' in data:
        if '/' in data['track']:
            album_kwargs['num_tracks'] = int(data['track'].split('/')[1])
            track_kwargs['track_no'] = int(data['track'].split('/')[0])
        else:
            track_kwargs['track_no'] = int(data['track'])

    if 'artist' in data:
        artist_kwargs['name'] = data['artist']
        albumartist_kwargs['name'] = data['artist']

    if 'albumartist' in data:
        albumartist_kwargs['name'] = data['albumartist']

    if 'album' in data:
        album_kwargs['name'] = data['album']

    if 'title' in data:
        track_kwargs['name'] = data['title']

    if 'musicbrainz_trackid' in data:
        track_kwargs['musicbrainz_id'] = data['musicbrainz_trackid']

    if 'musicbrainz_albumid' in data:
        album_kwargs['musicbrainz_id'] = data['musicbrainz_albumid']

    if 'musicbrainz_artistid' in data:
        artist_kwargs['musicbrainz_id'] = data['musicbrainz_artistid']

    if 'musicbrainz_albumartistid' in data:
        albumartist_kwargs['musicbrainz_id'] = data['musicbrainz_albumartistid']

    if data['file'][0] == '/':
        path = data['file'][1:]
    else:
        path = data['file']

    if artist_kwargs:
        artist = Artist(**artist_kwargs)
        track_kwargs['artists'] = [artist]

    if albumartist_kwargs:
        albumartist = Artist(**albumartist_kwargs)
        album_kwargs['artists'] = [albumartist]
    
    if album_kwargs:
        album = Album(**album_kwargs)
        track_kwargs['album'] = album

    track_kwargs['uri'] = path_to_uri(music_dir, path)
    track_kwargs['length'] = int(data.get('time', 0)) * 1000

    track = Track(**track_kwargs)
    tracks.add(track)



import sys

from mopidy import settings
from mopidy.backends.local import LocalBackend
from mopidy.models import Track

from tests import unittest
from tests.backends.base.current_playlist import CurrentPlaylistControllerTest
from tests.backends.local import generate_song


@unittest.skipIf(sys.platform == 'win32',
    'Our Windows build server does not support GStreamer yet')
class LocalCurrentPlaylistControllerTest(CurrentPlaylistControllerTest,
        unittest.TestCase):

    backend_class = LocalBackend
    tracks = [Track(uri=generate_song(i), length=4464)
        for i in range(1, 4)]

    def setUp(self):
        settings.BACKENDS = ('mopidy.backends.local.LocalBackend',)
        super(LocalCurrentPlaylistControllerTest, self).setUp()

    def tearDown(self):
        super(LocalCurrentPlaylistControllerTest, self).tearDown()
        settings.runtime.clear()



from mopidy.models import Track

from tests.frontends.mpd import protocol


class StatusHandlerTest(protocol.BaseTestCase):
    def test_clearerror(self):
        self.sendRequest(u'clearerror')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_currentsong(self):
        track = Track()
        self.backend.current_playlist.append([track])
        self.backend.playback.play()
        self.sendRequest(u'currentsong')
        self.assertInResponse(u'file: ')
        self.assertInResponse(u'Time: 0')
        self.assertInResponse(u'Artist: ')
        self.assertInResponse(u'Title: ')
        self.assertInResponse(u'Album: ')
        self.assertInResponse(u'Track: 0')
        self.assertInResponse(u'Date: ')
        self.assertInResponse(u'Pos: 0')
        self.assertInResponse(u'Id: 0')
        self.assertInResponse(u'OK')

    def test_currentsong_without_song(self):
        self.sendRequest(u'currentsong')
        self.assertInResponse(u'OK')

    def test_stats_command(self):
        self.sendRequest(u'stats')
        self.assertInResponse(u'OK')

    def test_status_command(self):
        self.sendRequest(u'status')
        self.assertInResponse(u'OK')



import pygst
pygst.require('0.10')
import gst

import logging

logger = logging.getLogger('mopidy.outputs')

class BaseOutput(object):
    """Base class for pluggable audio outputs."""

    MESSAGE_EOS = gst.MESSAGE_EOS
    MESSAGE_ERROR = gst.MESSAGE_ERROR
    MESSAGE_WARNING = gst.MESSAGE_WARNING

    def __init__(self, gstreamer):
        self.gstreamer = gstreamer
        self.bin = self._build_bin()
        self.bin.set_name(self.get_name())

        self.modify_bin()

    def _build_bin(self):
        description = 'queue ! %s' % self.describe_bin()
        logger.debug('Creating new output: %s', description)
        return gst.parse_bin_from_description(description, True)

    def connect(self):
        """Attach output to GStreamer pipeline."""
        self.gstreamer.connect_output(self.bin)
        self.on_connect()

    def on_connect(self):
        """
        Called after output has been connected to GStreamer pipeline.

        *MAY be implemented by subclass.*
        """
        pass

    def remove(self):
        """Remove output from GStreamer pipeline."""
        self.gstreamer.remove_output(self.bin)
        self.on_remove()

    def on_remove(self):
        """
        Called after output has been removed from GStreamer pipeline.

        *MAY be implemented by subclass.*
        """
        pass

    def get_name(self):
        """
        Get name of the output. Defaults to the output's class name.

        *MAY be implemented by subclass.*

        :rtype: string
        """
        return self.__class__.__name__

    def modify_bin(self):
        """
        Modifies ``self.bin`` before it is installed if needed.

        Overriding this method allows for outputs to modify the constructed bin
        before it is installed. This can for instance be a good place to call
        `set_properties` on elements that need to be configured.

        *MAY be implemented by subclass.*
        """
        pass

    def describe_bin(self):
        """
        Return string describing the output bin in :command:`gst-launch`
        format.

        For simple cases this can just be a sink such as ``autoaudiosink``,
        or it can be a chain like ``element1 ! element2 ! sink``. See the
        manpage of :command:`gst-launch` for details on the format.

        *MUST be implemented by subclass.*

        :rtype: string
        """
        raise NotImplementedError

    def set_properties(self, element, properties):
        """
        Helper method for setting of properties on elements.

        Will call :meth:`gst.Element.set_property` on ``element`` for each key
        in ``properties`` that has a value that is not :class:`None`.

        :param element: element to set properties on
        :type element: :class:`gst.Element`
        :param properties: properties to set on element
        :type properties: dict
        """
        for key, value in properties.items():
            if value is not None:
                element.set_property(key, value)



import datetime as dt
import logging

from spotify import Link, SpotifyError

from mopidy import settings
from mopidy.models import Artist, Album, Track, Playlist

logger = logging.getLogger('mopidy.backends.spotify.translator')

class SpotifyTranslator(object):
    @classmethod
    def to_mopidy_artist(cls, spotify_artist):
        if not spotify_artist.is_loaded():
            return Artist(name=u'[loading...]')
        return Artist(
            uri=str(Link.from_artist(spotify_artist)),
            name=spotify_artist.name()
        )

    @classmethod
    def to_mopidy_album(cls, spotify_album):
        if spotify_album is None or not spotify_album.is_loaded():
            return Album(name=u'[loading...]')
        # TODO pyspotify got much more data on albums than this
        return Album(name=spotify_album.name())

    @classmethod
    def to_mopidy_track(cls, spotify_track):
        uri = str(Link.from_track(spotify_track, 0))
        if not spotify_track.is_loaded():
            return Track(uri=uri, name=u'[loading...]')
        spotify_album = spotify_track.album()
        if (spotify_album is not None and spotify_album.is_loaded()
                and dt.MINYEAR <= int(spotify_album.year()) <= dt.MAXYEAR):
            date = dt.date(spotify_album.year(), 1, 1)
        else:
            date = None
        return Track(
            uri=uri,
            name=spotify_track.name(),
            artists=[cls.to_mopidy_artist(a) for a in spotify_track.artists()],
            album=cls.to_mopidy_album(spotify_track.album()),
            track_no=spotify_track.index(),
            date=date,
            length=spotify_track.duration(),
            bitrate=settings.SPOTIFY_BITRATE,
        )

    @classmethod
    def to_mopidy_playlist(cls, spotify_playlist):
        if not spotify_playlist.is_loaded():
            return Playlist(name=u'[loading...]')
        if spotify_playlist.type() != 'playlist':
            return
        try:
            return Playlist(
                uri=str(Link.from_playlist(spotify_playlist)),
                name=spotify_playlist.name(),
                # FIXME if check on link is a hackish workaround for is_local
                tracks=[cls.to_mopidy_track(t) for t in spotify_playlist
                    if str(Link.from_track(t, 0))],
            )
        except SpotifyError, e:
            logger.warning(u'Failed translating Spotify playlist: %s', e)



import logging
import Queue

from spotify import Link, SpotifyError

from mopidy.backends.base import BaseLibraryProvider
from mopidy.backends.spotify.translator import SpotifyTranslator
from mopidy.models import Playlist

logger = logging.getLogger('mopidy.backends.spotify.library')

class SpotifyLibraryProvider(BaseLibraryProvider):
    def find_exact(self, **query):
        return self.search(**query)

    def lookup(self, uri):
        try:
            spotify_track = Link.from_string(uri).as_track()
            # TODO Block until metadata_updated callback is called. Before that
            # the track will be unloaded, unless it's already in the stored
            # playlists.
            return SpotifyTranslator.to_mopidy_track(spotify_track)
        except SpotifyError as e:
            logger.debug(u'Failed to lookup "%s": %s', uri, e)
            return None

    def refresh(self, uri=None):
        pass # TODO

    def search(self, **query):
        if not query:
            # Since we can't search for the entire Spotify library, we return
            # all tracks in the stored playlists when the query is empty.
            tracks = []
            for playlist in self.backend.stored_playlists.playlists:
                tracks += playlist.tracks
            return Playlist(tracks=tracks)
        spotify_query = []
        for (field, values) in query.iteritems():
            if field == u'track':
                field = u'title'
            if field == u'date':
                field = u'year'
            if not hasattr(values, '__iter__'):
                values = [values]
            for value in values:
                if field == u'any':
                    spotify_query.append(value)
                elif field == u'year':
                    value = int(value.split('-')[0]) # Extract year
                    spotify_query.append(u'%s:%d' % (field, value))
                else:
                    spotify_query.append(u'%s:"%s"' % (field, value))
        spotify_query = u' '.join(spotify_query)
        logger.debug(u'Spotify search query: %s' % spotify_query)
        queue = Queue.Queue()
        self.backend.spotify.search(spotify_query, queue)
        try:
            return queue.get(timeout=3) # XXX What is an reasonable timeout?
        except Queue.Empty:
            return Playlist(tracks=[])



from collections import namedtuple

class ImmutableObject(object):
    """
    Superclass for immutable objects whose fields can only be modified via the
    constructor.

    :param kwargs: kwargs to set as fields on the object
    :type kwargs: any
    """

    def __init__(self, *args, **kwargs):
        for key, value in kwargs.items():
            if not hasattr(self, key):
                raise TypeError('__init__() got an unexpected keyword ' + \
                    'argument \'%s\'' % key)
            self.__dict__[key] = value

    def __setattr__(self, name, value):
        if name.startswith('_'):
            return super(ImmutableObject, self).__setattr__(name, value)
        raise AttributeError('Object is immutable.')

    def __repr__(self):
        kwarg_pairs = []
        for (key, value) in sorted(self.__dict__.items()):
            if isinstance(value, (frozenset, tuple)):
                value = list(value)
            kwarg_pairs.append('%s=%s' % (key, repr(value)))
        return '%(classname)s(%(kwargs)s)' % {
            'classname': self.__class__.__name__,
            'kwargs': ', '.join(kwarg_pairs),
        }

    def __hash__(self):
        hash_sum = 0
        for key, value in self.__dict__.items():
            hash_sum += hash(key) + hash(value)
        return hash_sum

    def __eq__(self, other):
        if not isinstance(other, self.__class__):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not self.__eq__(other)

    def copy(self, **values):
        """
        Copy the model with ``field`` updated to new value.

        Examples::

            # Returns a track with a new name
            Track(name='foo').copy(name='bar')
            # Return an album with a new number of tracks
            Album(num_tracks=2).copy(num_tracks=5)

        :param values: the model fields to modify
        :type values: dict
        :rtype: new instance of the model being copied
        """
        data = {}
        for key in self.__dict__.keys():
            public_key = key.lstrip('_')
            data[public_key] = values.pop(public_key, self.__dict__[key])
        for key in values.keys():
            if hasattr(self, key):
                data[key] = values.pop(key)
        if values:
            raise TypeError("copy() got an unexpected keyword argument '%s'"
                % key)
        return self.__class__(**data)


class Artist(ImmutableObject):
    """
    :param uri: artist URI
    :type uri: string
    :param name: artist name
    :type name: string
    :param musicbrainz_id: MusicBrainz ID
    :type musicbrainz_id: string
    """

    #: The artist URI. Read-only.
    uri = None

    #: The artist name. Read-only.
    name = None

    #: The MusicBrainz ID of the artist. Read-only.
    musicbrainz_id = None


class Album(ImmutableObject):
    """
    :param uri: album URI
    :type uri: string
    :param name: album name
    :type name: string
    :param artists: album artists
    :type artists: list of :class:`Artist`
    :param num_tracks: number of tracks in album
    :type num_tracks: integer
    :param musicbrainz_id: MusicBrainz ID
    :type musicbrainz_id: string
    """

    #: The album URI. Read-only.
    uri = None

    #: The album name. Read-only.
    name = None

    #: A set of album artists. Read-only.
    artists = frozenset()

    #: The number of tracks in the album. Read-only.
    num_tracks = 0

    #: The MusicBrainz ID of the album. Read-only.
    musicbrainz_id = None

    def __init__(self, *args, **kwargs):
        self.__dict__['artists'] = frozenset(kwargs.pop('artists', []))
        super(Album, self).__init__(*args, **kwargs)


CpTrack = namedtuple('CpTrack', ['cpid', 'track'])


class Track(ImmutableObject):
    """
    :param uri: track URI
    :type uri: string
    :param name: track name
    :type name: string
    :param artists: track artists
    :type artists: list of :class:`Artist`
    :param album: track album
    :type album: :class:`Album`
    :param track_no: track number in album
    :type track_no: integer
    :param date: track release date
    :type date: :class:`datetime.date`
    :param length: track length in milliseconds
    :type length: integer
    :param bitrate: bitrate in kbit/s
    :type bitrate: integer
    :param musicbrainz_id: MusicBrainz ID
    :type musicbrainz_id: string
    """

    #: The track URI. Read-only.
    uri = None

    #: The track name. Read-only.
    name = None

    #: A set of track artists. Read-only.
    artists = frozenset()

    #: The track :class:`Album`. Read-only.
    album = None

    #: The track number in album. Read-only.
    track_no = 0

    #: The track release date. Read-only.
    date = None

    #: The track length in milliseconds. Read-only.
    length = None

    #: The track's bitrate in kbit/s. Read-only.
    bitrate = None

    #: The MusicBrainz ID of the track. Read-only.
    musicbrainz_id = None

    def __init__(self, *args, **kwargs):
        self.__dict__['artists'] = frozenset(kwargs.pop('artists', []))
        super(Track, self).__init__(*args, **kwargs)


class Playlist(ImmutableObject):
    """
        :param uri: playlist URI
        :type uri: string
        :param name: playlist name
        :type name: string
        :param tracks: playlist's tracks
        :type tracks: list of :class:`Track` elements
        :param last_modified: playlist's modification time
        :type last_modified: :class:`datetime.datetime`
    """

    #: The playlist URI. Read-only.
    uri = None

    #: The playlist name. Read-only.
    name = None

    #: The playlist's tracks. Read-only.
    tracks = tuple()

    #: The playlist modification time. Read-only.
    #:
    #: :class:`datetime.datetime`, or :class:`None` if unknown.
    last_modified = None

    def __init__(self, *args, **kwargs):
        self.__dict__['tracks'] = tuple(kwargs.pop('tracks', []))
        super(Playlist, self).__init__(*args, **kwargs)

    @property
    def length(self):
        """The number of tracks in the playlist. Read-only."""
        return len(self.tracks)



import mock
import random

from mopidy.models import CpTrack, Playlist, Track
from mopidy.gstreamer import GStreamer

from tests.backends.base import populate_playlist


class CurrentPlaylistControllerTest(object):
    tracks = []

    def setUp(self):
        self.backend = self.backend_class()
        self.backend.gstreamer = mock.Mock(spec=GStreamer)
        self.controller = self.backend.current_playlist
        self.playback = self.backend.playback

        assert len(self.tracks) == 3, 'Need three tracks to run tests.'

    def test_length(self):
        self.assertEqual(0, len(self.controller.cp_tracks))
        self.assertEqual(0, self.controller.length)
        self.controller.append(self.tracks)
        self.assertEqual(3, len(self.controller.cp_tracks))
        self.assertEqual(3, self.controller.length)

    def test_add(self):
        for track in self.tracks:
            cp_track = self.controller.add(track)
            self.assertEqual(track, self.controller.tracks[-1])
            self.assertEqual(cp_track, self.controller.cp_tracks[-1])
            self.assertEqual(track, cp_track.track)

    def test_add_at_position(self):
        for track in self.tracks[:-1]:
            cp_track = self.controller.add(track, 0)
            self.assertEqual(track, self.controller.tracks[0])
            self.assertEqual(cp_track, self.controller.cp_tracks[0])
            self.assertEqual(track, cp_track.track)

    @populate_playlist
    def test_add_at_position_outside_of_playlist(self):
        test = lambda: self.controller.add(self.tracks[0], len(self.tracks)+2)
        self.assertRaises(AssertionError, test)

    @populate_playlist
    def test_get_by_cpid(self):
        cp_track = self.controller.cp_tracks[1]
        self.assertEqual(cp_track, self.controller.get(cpid=cp_track.cpid))

    @populate_playlist
    def test_get_by_uri(self):
        cp_track = self.controller.cp_tracks[1]
        self.assertEqual(cp_track, self.controller.get(uri=cp_track.track.uri))

    @populate_playlist
    def test_get_by_uri_raises_error_for_invalid_uri(self):
        test = lambda: self.controller.get(uri='foobar')
        self.assertRaises(LookupError, test)

    @populate_playlist
    def test_clear(self):
        self.controller.clear()
        self.assertEqual(len(self.controller.tracks), 0)

    def test_clear_empty_playlist(self):
        self.controller.clear()
        self.assertEqual(len(self.controller.tracks), 0)

    @populate_playlist
    def test_clear_when_playing(self):
        self.playback.play()
        self.assertEqual(self.playback.state, self.playback.PLAYING)
        self.controller.clear()
        self.assertEqual(self.playback.state, self.playback.STOPPED)

    def test_get_by_uri_returns_unique_match(self):
        track = Track(uri='a')
        self.controller.append([Track(uri='z'), track, Track(uri='y')])
        self.assertEqual(track, self.controller.get(uri='a')[1])

    def test_get_by_uri_raises_error_if_multiple_matches(self):
        track = Track(uri='a')
        self.controller.append([Track(uri='z'), track, track])
        try:
            self.controller.get(uri='a')
            self.fail(u'Should raise LookupError if multiple matches')
        except LookupError as e:
            self.assertEqual(u'"uri=a" match multiple tracks', e[0])

    def test_get_by_uri_raises_error_if_no_match(self):
        self.controller.playlist = Playlist(
            tracks=[Track(uri='z'), Track(uri='y')])
        try:
            self.controller.get(uri='a')
            self.fail(u'Should raise LookupError if no match')
        except LookupError as e:
            self.assertEqual(u'"uri=a" match no tracks', e[0])

    def test_get_by_multiple_criteria_returns_elements_matching_all(self):
        track1 = Track(uri='a', name='x')
        track2 = Track(uri='b', name='x')
        track3 = Track(uri='b', name='y')
        self.controller.append([track1, track2, track3])
        self.assertEqual(track1, self.controller.get(uri='a', name='x')[1])
        self.assertEqual(track2, self.controller.get(uri='b', name='x')[1])
        self.assertEqual(track3, self.controller.get(uri='b', name='y')[1])

    def test_get_by_criteria_that_is_not_present_in_all_elements(self):
        track1 = Track()
        track2 = Track(uri='b')
        track3 = Track()
        self.controller.append([track1, track2, track3])
        self.assertEqual(track2, self.controller.get(uri='b')[1])

    def test_append_appends_to_the_current_playlist(self):
        self.controller.append([Track(uri='a'), Track(uri='b')])
        self.assertEqual(len(self.controller.tracks), 2)
        self.controller.append([Track(uri='c'), Track(uri='d')])
        self.assertEqual(len(self.controller.tracks), 4)
        self.assertEqual(self.controller.tracks[0].uri, 'a')
        self.assertEqual(self.controller.tracks[1].uri, 'b')
        self.assertEqual(self.controller.tracks[2].uri, 'c')
        self.assertEqual(self.controller.tracks[3].uri, 'd')

    def test_append_does_not_reset_version(self):
        version = self.controller.version
        self.controller.append([])
        self.assertEqual(self.controller.version, version)

    @populate_playlist
    def test_append_preserves_playing_state(self):
        self.playback.play()
        track = self.playback.current_track
        self.controller.append(self.controller.tracks[1:2])
        self.assertEqual(self.playback.state, self.playback.PLAYING)
        self.assertEqual(self.playback.current_track, track)

    @populate_playlist
    def test_append_preserves_stopped_state(self):
        self.controller.append(self.controller.tracks[1:2])
        self.assertEqual(self.playback.state, self.playback.STOPPED)
        self.assertEqual(self.playback.current_track, None)

    def test_index_returns_index_of_track(self):
        cp_tracks = []
        for track in self.tracks:
            cp_tracks.append(self.controller.add(track))
        self.assertEquals(0, self.controller.index(cp_tracks[0]))
        self.assertEquals(1, self.controller.index(cp_tracks[1]))
        self.assertEquals(2, self.controller.index(cp_tracks[2]))

    def test_index_raises_value_error_if_item_not_found(self):
        test = lambda: self.controller.index(CpTrack(0, Track()))
        self.assertRaises(ValueError, test)

    @populate_playlist
    def test_move_single(self):
        self.controller.move(0, 0, 2)

        tracks = self.controller.tracks
        self.assertEqual(tracks[2], self.tracks[0])

    @populate_playlist
    def test_move_group(self):
        self.controller.move(0, 2, 1)

        tracks = self.controller.tracks
        self.assertEqual(tracks[1], self.tracks[0])
        self.assertEqual(tracks[2], self.tracks[1])

    @populate_playlist
    def test_moving_track_outside_of_playlist(self):
        tracks = len(self.controller.tracks)
        test = lambda: self.controller.move(0, 0, tracks+5)
        self.assertRaises(AssertionError, test)

    @populate_playlist
    def test_move_group_outside_of_playlist(self):
        tracks = len(self.controller.tracks)
        test = lambda: self.controller.move(0, 2, tracks+5)
        self.assertRaises(AssertionError, test)

    @populate_playlist
    def test_move_group_out_of_range(self):
        tracks = len(self.controller.tracks)
        test = lambda: self.controller.move(tracks+2, tracks+3, 0)
        self.assertRaises(AssertionError, test)

    @populate_playlist
    def test_move_group_invalid_group(self):
        test = lambda: self.controller.move(2, 1, 0)
        self.assertRaises(AssertionError, test)

    def test_tracks_attribute_is_immutable(self):
        tracks1 = self.controller.tracks
        tracks2 = self.controller.tracks
        self.assertNotEqual(id(tracks1), id(tracks2))

    @populate_playlist
    def test_remove(self):
        track1 = self.controller.tracks[1]
        track2 = self.controller.tracks[2]
        version = self.controller.version
        self.controller.remove(uri=track1.uri)
        self.assert_(version < self.controller.version)
        self.assert_(track1 not in self.controller.tracks)
        self.assertEqual(track2, self.controller.tracks[1])

    @populate_playlist
    def test_removing_track_that_does_not_exist(self):
        test = lambda: self.controller.remove(uri='/nonexistant')
        self.assertRaises(LookupError, test)

    def test_removing_from_empty_playlist(self):
        test = lambda: self.controller.remove(uri='/nonexistant')
        self.assertRaises(LookupError, test)

    @populate_playlist
    def test_shuffle(self):
        random.seed(1)
        self.controller.shuffle()

        shuffled_tracks = self.controller.tracks

        self.assertNotEqual(self.tracks, shuffled_tracks)
        self.assertEqual(set(self.tracks), set(shuffled_tracks))

    @populate_playlist
    def test_shuffle_subset(self):
        random.seed(1)
        self.controller.shuffle(1, 3)

        shuffled_tracks = self.controller.tracks

        self.assertNotEqual(self.tracks, shuffled_tracks)
        self.assertEqual(self.tracks[0], shuffled_tracks[0])
        self.assertEqual(set(self.tracks), set(shuffled_tracks))

    @populate_playlist
    def test_shuffle_invalid_subset(self):
        test = lambda: self.controller.shuffle(3, 1)
        self.assertRaises(AssertionError, test)

    @populate_playlist
    def test_shuffle_superset(self):
        tracks = len(self.controller.tracks)
        test = lambda: self.controller.shuffle(1, tracks+5)
        self.assertRaises(AssertionError, test)

    @populate_playlist
    def test_shuffle_open_subset(self):
        random.seed(1)
        self.controller.shuffle(1)

        shuffled_tracks = self.controller.tracks

        self.assertNotEqual(self.tracks, shuffled_tracks)
        self.assertEqual(self.tracks[0], shuffled_tracks[0])
        self.assertEqual(set(self.tracks), set(shuffled_tracks))

    @populate_playlist
    def test_slice_returns_a_subset_of_tracks(self):
        track_slice = self.controller.slice(1, 3)
        self.assertEqual(2, len(track_slice))
        self.assertEqual(self.tracks[1], track_slice[0].track)
        self.assertEqual(self.tracks[2], track_slice[1].track)

    @populate_playlist
    def test_slice_returns_empty_list_if_indexes_outside_tracks_list(self):
        self.assertEqual(0, len(self.controller.slice(7, 8)))
        self.assertEqual(0, len(self.controller.slice(-1, 1)))

    def test_version_does_not_change_when_appending_nothing(self):
        version = self.controller.version
        self.controller.append([])
        self.assertEquals(version, self.controller.version)

    def test_version_increases_when_appending_something(self):
        version = self.controller.version
        self.controller.append([Track()])
        self.assert_(version < self.controller.version)



# Add ../ to the path so we can run Mopidy from a Git checkout without
# installing it on the system.
import os
import sys
sys.path.insert(0,
    os.path.abspath(os.path.join(os.path.dirname(__file__), '../')))

if __name__ == '__main__':
    from mopidy.core import main
    main()



import logging

from .current_playlist import CurrentPlaylistController
from .library import LibraryController, BaseLibraryProvider
from .playback import PlaybackController, BasePlaybackProvider
from .stored_playlists import (StoredPlaylistsController,
    BaseStoredPlaylistsProvider)

logger = logging.getLogger('mopidy.backends.base')

class Backend(object):
    #: The current playlist controller. An instance of
    #: :class:`mopidy.backends.base.CurrentPlaylistController`.
    current_playlist = None

    #: The library controller. An instance of
    # :class:`mopidy.backends.base.LibraryController`.
    library = None

    #: The playback controller. An instance of
    #: :class:`mopidy.backends.base.PlaybackController`.
    playback = None

    #: The stored playlists controller. An instance of
    #: :class:`mopidy.backends.base.StoredPlaylistsController`.
    stored_playlists = None

    #: List of URI schemes this backend can handle.
    uri_schemes = []



import mock

from pykka.registry import ActorRegistry

from mopidy.backends.dummy import DummyBackend
from mopidy.listeners import BackendListener
from mopidy.models import Track

from tests import unittest


@mock.patch.object(BackendListener, 'send')
class BackendEventsTest(unittest.TestCase):
    def setUp(self):
        self.backend = DummyBackend.start().proxy()

    def tearDown(self):
        ActorRegistry.stop_all()

    def test_pause_sends_track_playback_paused_event(self, send):
        self.backend.current_playlist.add(Track(uri='a'))
        self.backend.playback.play().get()
        send.reset_mock()
        self.backend.playback.pause().get()
        self.assertEqual(send.call_args[0][0], 'track_playback_paused')

    def test_resume_sends_track_playback_resumed(self, send):
        self.backend.current_playlist.add(Track(uri='a'))
        self.backend.playback.play()
        self.backend.playback.pause().get()
        send.reset_mock()
        self.backend.playback.resume().get()
        self.assertEqual(send.call_args[0][0], 'track_playback_resumed')

    def test_play_sends_track_playback_started_event(self, send):
        self.backend.current_playlist.add(Track(uri='a'))
        send.reset_mock()
        self.backend.playback.play().get()
        self.assertEqual(send.call_args[0][0], 'track_playback_started')

    def test_stop_sends_track_playback_ended_event(self, send):
        self.backend.current_playlist.add(Track(uri='a'))
        self.backend.playback.play().get()
        send.reset_mock()
        self.backend.playback.stop().get()
        self.assertEqual(send.call_args_list[0][0][0], 'track_playback_ended')

    def test_seek_sends_seeked_event(self, send):
        self.backend.current_playlist.add(Track(uri='a', length=40000))
        self.backend.playback.play().get()
        send.reset_mock()
        self.backend.playback.seek(1000).get()
        self.assertEqual(send.call_args[0][0], 'seeked')



import logging
import optparse
import os
import signal
import sys

import gobject
gobject.threads_init()

# Extract any non-GStreamer arguments, and leave the GStreamer arguments for
# processing by GStreamer. This needs to be done before GStreamer is imported,
# so that GStreamer doesn't hijack e.g. ``--help``.
# NOTE This naive fix does not support values like ``bar`` in
# ``--gst-foo bar``. Use equals to pass values, like ``--gst-foo=bar``.
def is_gst_arg(argument):
    return argument.startswith('--gst') or argument == '--help-gst'
gstreamer_args = [arg for arg in sys.argv[1:] if is_gst_arg(arg)]
mopidy_args = [arg for arg in sys.argv[1:] if not is_gst_arg(arg)]
sys.argv[1:] = gstreamer_args

from mopidy import (get_version, settings, OptionalDependencyError,
    SettingsError, DATA_PATH, SETTINGS_PATH, SETTINGS_FILE)
from mopidy.gstreamer import GStreamer
from mopidy.utils import get_class
from mopidy.utils.log import setup_logging
from mopidy.utils.path import get_or_create_folder, get_or_create_file
from mopidy.utils.process import (exit_handler, stop_remaining_actors,
    stop_actors_by_class)
from mopidy.utils.settings import list_settings_optparse_callback

logger = logging.getLogger('mopidy.core')

def main():
    signal.signal(signal.SIGTERM, exit_handler)
    loop = gobject.MainLoop()
    try:
        options = parse_options()
        setup_logging(options.verbosity_level, options.save_debug_log)
        check_old_folders()
        setup_settings(options.interactive)
        setup_gstreamer()
        setup_mixer()
        setup_backend()
        setup_frontends()
        loop.run()
    except SettingsError as e:
        logger.error(e.message)
    except KeyboardInterrupt:
        logger.info(u'Interrupted. Exiting...')
    except Exception as e:
        logger.exception(e)
    finally:
        loop.quit()
        stop_frontends()
        stop_backend()
        stop_mixer()
        stop_gstreamer()
        stop_remaining_actors()

def parse_options():
    parser = optparse.OptionParser(version=u'Mopidy %s' % get_version())
    parser.add_option('--help-gst',
        action='store_true', dest='help_gst',
        help='show GStreamer help options')
    parser.add_option('-i', '--interactive',
        action='store_true', dest='interactive',
        help='ask interactively for required settings which are missing')
    parser.add_option('-q', '--quiet',
        action='store_const', const=0, dest='verbosity_level',
        help='less output (warning level)')
    parser.add_option('-v', '--verbose',
        action='count', default=1, dest='verbosity_level',
        help='more output (debug level)')
    parser.add_option('--save-debug-log',
        action='store_true', dest='save_debug_log',
        help='save debug log to "./mopidy.log"')
    parser.add_option('--list-settings',
        action='callback', callback=list_settings_optparse_callback,
        help='list current settings')
    return parser.parse_args(args=mopidy_args)[0]

def check_old_folders():
    old_settings_folder = os.path.expanduser(u'~/.mopidy')

    if not os.path.isdir(old_settings_folder):
        return

    logger.warning(u'Old settings folder found at %s, settings.py should be '
        'moved to %s, any cache data should be deleted. See release notes '
        'for further instructions.', old_settings_folder, SETTINGS_PATH)

def setup_settings(interactive):
    get_or_create_folder(SETTINGS_PATH)
    get_or_create_folder(DATA_PATH)
    get_or_create_file(SETTINGS_FILE)
    try:
        settings.validate(interactive)
    except SettingsError, e:
        logger.error(e.message)
        sys.exit(1)

def setup_gstreamer():
    GStreamer.start()

def stop_gstreamer():
    stop_actors_by_class(GStreamer)

def setup_mixer():
    get_class(settings.MIXER).start()

def stop_mixer():
    stop_actors_by_class(get_class(settings.MIXER))

def setup_backend():
    get_class(settings.BACKENDS[0]).start()

def stop_backend():
    stop_actors_by_class(get_class(settings.BACKENDS[0]))

def setup_frontends():
    for frontend_class_name in settings.FRONTENDS:
        try:
            get_class(frontend_class_name).start()
        except OptionalDependencyError as e:
            logger.info(u'Disabled: %s (%s)', frontend_class_name, e)

def stop_frontends():
    for frontend_class_name in settings.FRONTENDS:
        try:
            stop_actors_by_class(get_class(frontend_class_name))
        except OptionalDependencyError:
            pass



import logging
import os
import threading

from spotify.manager import SpotifySessionManager as PyspotifySessionManager

from pykka.registry import ActorRegistry

from mopidy import get_version, settings, CACHE_PATH
from mopidy.backends.base import Backend
from mopidy.backends.spotify import BITRATES
from mopidy.backends.spotify.container_manager import SpotifyContainerManager
from mopidy.backends.spotify.playlist_manager import SpotifyPlaylistManager
from mopidy.backends.spotify.translator import SpotifyTranslator
from mopidy.models import Playlist
from mopidy.gstreamer import GStreamer
from mopidy.utils.process import BaseThread

logger = logging.getLogger('mopidy.backends.spotify.session_manager')

# pylint: disable = R0901
# SpotifySessionManager: Too many ancestors (9/7)


class SpotifySessionManager(BaseThread, PyspotifySessionManager):
    cache_location = settings.SPOTIFY_CACHE_PATH or CACHE_PATH
    settings_location = settings.SPOTIFY_CACHE_PATH or CACHE_PATH
    appkey_file = os.path.join(os.path.dirname(__file__), 'spotify_appkey.key')
    user_agent = 'Mopidy %s' % get_version()

    def __init__(self, username, password):
        PyspotifySessionManager.__init__(self, username, password)
        BaseThread.__init__(self)
        self.name = 'SpotifyThread'

        self.gstreamer = None
        self.backend = None

        self.connected = threading.Event()
        self.session = None

        self.container_manager = None
        self.playlist_manager = None

    def run_inside_try(self):
        self.setup()
        self.connect()

    def setup(self):
        gstreamer_refs = ActorRegistry.get_by_class(GStreamer)
        assert len(gstreamer_refs) == 1, \
            'Expected exactly one running gstreamer.'
        self.gstreamer = gstreamer_refs[0].proxy()

        backend_refs = ActorRegistry.get_by_class(Backend)
        assert len(backend_refs) == 1, 'Expected exactly one running backend.'
        self.backend = backend_refs[0].proxy()

    def logged_in(self, session, error):
        """Callback used by pyspotify"""
        if error:
            logger.error(u'Spotify login error: %s', error)
            return

        logger.info(u'Connected to Spotify')
        self.session = session

        logger.debug(u'Preferred Spotify bitrate is %s kbps',
            settings.SPOTIFY_BITRATE)
        self.session.set_preferred_bitrate(BITRATES[settings.SPOTIFY_BITRATE])

        self.container_manager = SpotifyContainerManager(self)
        self.playlist_manager = SpotifyPlaylistManager(self)

        self.container_manager.watch(self.session.playlist_container())

        self.connected.set()

    def logged_out(self, session):
        """Callback used by pyspotify"""
        logger.info(u'Disconnected from Spotify')

    def metadata_updated(self, session):
        """Callback used by pyspotify"""
        logger.debug(u'Callback called: Metadata updated')

    def connection_error(self, session, error):
        """Callback used by pyspotify"""
        if error is None:
            logger.info(u'Spotify connection OK')
        else:
            logger.error(u'Spotify connection error: %s', error)
            self.backend.playback.pause()

    def message_to_user(self, session, message):
        """Callback used by pyspotify"""
        logger.debug(u'User message: %s', message.strip())

    def music_delivery(self, session, frames, frame_size, num_frames,
            sample_type, sample_rate, channels):
        """Callback used by pyspotify"""
        # pylint: disable = R0913
        # Too many arguments (8/5)
        assert sample_type == 0, u'Expects 16-bit signed integer samples'
        capabilites = """
            audio/x-raw-int,
            endianness=(int)1234,
            channels=(int)%(channels)d,
            width=(int)16,
            depth=(int)16,
            signed=(boolean)true,
            rate=(int)%(sample_rate)d
        """ % {
            'sample_rate': sample_rate,
            'channels': channels,
        }
        self.gstreamer.emit_data(capabilites, bytes(frames))
        return num_frames

    def play_token_lost(self, session):
        """Callback used by pyspotify"""
        logger.debug(u'Play token lost')
        self.backend.playback.pause()

    def log_message(self, session, data):
        """Callback used by pyspotify"""
        logger.debug(u'System message: %s' % data.strip())

    def end_of_track(self, session):
        """Callback used by pyspotify"""
        logger.debug(u'End of data stream reached')
        self.gstreamer.emit_end_of_stream()

    def refresh_stored_playlists(self):
        """Refresh the stored playlists in the backend with fresh meta data
        from Spotify"""
        playlists = map(SpotifyTranslator.to_mopidy_playlist,
            self.session.playlist_container())
        playlists = filter(None, playlists)
        self.backend.stored_playlists.playlists = playlists
        logger.debug(u'Refreshed %d stored playlist(s)', len(playlists))

    def search(self, query, queue):
        """Search method used by Mopidy backend"""
        def callback(results, userdata=None):
            # TODO Include results from results.albums(), etc. too
            # TODO Consider launching a second search if results.total_tracks()
            # is larger than len(results.tracks())
            playlist = Playlist(tracks=[
                SpotifyTranslator.to_mopidy_track(t)
                for t in results.tracks()])
            queue.put(playlist)
        self.connected.wait()
        self.session.search(query, callback, track_count=100,
            album_count=0, artist_count=0)

    def logout(self):
        """Log out from spotify"""
        logger.debug(u'Logging out from Spotify')
        if self.session:
            self.session.logout()



import datetime
import logging

from spotify.manager import SpotifyPlaylistManager as PyspotifyPlaylistManager

logger = logging.getLogger('mopidy.backends.spotify.playlist_manager')

class SpotifyPlaylistManager(PyspotifyPlaylistManager):
    def __init__(self, session_manager):
        PyspotifyPlaylistManager.__init__(self)
        self.session_manager = session_manager

    def tracks_added(self, playlist, tracks, position, userdata):
        """Callback used by pyspotify"""
        logger.debug(u'Callback called: '
            u'%d track(s) added to position %d in playlist "%s"',
            len(tracks), position, playlist.name())
        self.session_manager.refresh_stored_playlists()

    def tracks_moved(self, playlist, tracks, new_position, userdata):
        """Callback used by pyspotify"""
        logger.debug(u'Callback called: '
            u'%d track(s) moved to position %d in playlist "%s"',
            len(tracks), new_position, playlist.name())
        self.session_manager.refresh_stored_playlists()

    def tracks_removed(self, playlist, tracks, userdata):
        """Callback used by pyspotify"""
        logger.debug(u'Callback called: '
            u'%d track(s) removed from playlist "%s"',
            len(tracks), playlist.name())
        self.session_manager.refresh_stored_playlists()

    def playlist_renamed(self, playlist, userdata):
        """Callback used by pyspotify"""
        logger.debug(u'Callback called: Playlist renamed to "%s"',
            playlist.name())
        self.session_manager.refresh_stored_playlists()

    def playlist_state_changed(self, playlist, userdata):
        """Callback used by pyspotify"""
        logger.debug(u'Callback called: The state of playlist "%s" changed',
            playlist.name())

    def playlist_update_in_progress(self, playlist, done, userdata):
        """Callback used by pyspotify"""
        if done:
            logger.debug(u'Callback called: '
                u'Update of playlist "%s" done', playlist.name())
        else:
            logger.debug(u'Callback called: '
                u'Update of playlist "%s" in progress', playlist.name())

    def playlist_metadata_updated(self, playlist, userdata):
        """Callback used by pyspotify"""
        logger.debug(u'Callback called: Metadata updated for playlist "%s"',
            playlist.name())

    def track_created_changed(self, playlist, position, user, when, userdata):
        """Callback used by pyspotify"""
        when = datetime.datetime.fromtimestamp(when)
        logger.debug(
            u'Callback called: Created by/when for track %d in playlist '
            u'"%s" changed to user "N/A" and time "%s"',
            position, playlist.name(), when)

    def track_message_changed(self, playlist, position, message, userdata):
        """Callback used by pyspotify"""
        logger.debug(
            u'Callback called: Message for track %d in playlist '
            u'"%s" changed to "%s"', position, playlist.name(), message)

    def track_seen_changed(self, playlist, position, seen, userdata):
        """Callback used by pyspotify"""
        logger.debug(
            u'Callback called: Seen attribute for track %d in playlist '
            u'"%s" changed to "%s"', position, playlist.name(), seen)

    def description_changed(self, playlist, description, userdata):
        """Callback used by pyspotify"""
        logger.debug(
            u'Callback called: Description changed for playlist "%s" to "%s"',
            playlist.name(), description)

    def subscribers_changed(self, playlist, userdata):
        """Callback used by pyspotify"""
        logger.debug(
            u'Callback called: Subscribers changed for playlist "%s"',
            playlist.name())

    def image_changed(self, playlist, image, userdata):
        """Callback used by pyspotify"""
        logger.debug(u'Callback called: Image changed for playlist "%s"',
            playlist.name())



import logging

logger = logging.getLogger('mopidy.backends.base')

class LibraryController(object):
    """
    :param backend: backend the controller is a part of
    :type backend: :class:`mopidy.backends.base.Backend`
    :param provider: provider the controller should use
    :type provider: instance of :class:`BaseLibraryProvider`
    """

    pykka_traversable = True

    def __init__(self, backend, provider):
        self.backend = backend
        self.provider = provider

    def find_exact(self, **query):
        """
        Search the library for tracks where ``field`` is ``values``.

        Examples::

            # Returns results matching 'a'
            find_exact(any=['a'])
            # Returns results matching artist 'xyz'
            find_exact(artist=['xyz'])
            # Returns results matching 'a' and 'b' and artist 'xyz'
            find_exact(any=['a', 'b'], artist=['xyz'])

        :param query: one or more queries to search for
        :type query: dict
        :rtype: :class:`mopidy.models.Playlist`
        """
        return self.provider.find_exact(**query)

    def lookup(self, uri):
        """
        Lookup track with given URI. Returns :class:`None` if not found.

        :param uri: track URI
        :type uri: string
        :rtype: :class:`mopidy.models.Track` or :class:`None`
        """
        return self.provider.lookup(uri)

    def refresh(self, uri=None):
        """
        Refresh library. Limit to URI and below if an URI is given.

        :param uri: directory or track URI
        :type uri: string
        """
        return self.provider.refresh(uri)

    def search(self, **query):
        """
        Search the library for tracks where ``field`` contains ``values``.

        Examples::

            # Returns results matching 'a'
            search(any=['a'])
            # Returns results matching artist 'xyz'
            search(artist=['xyz'])
            # Returns results matching 'a' and 'b' and artist 'xyz'
            search(any=['a', 'b'], artist=['xyz'])

        :param query: one or more queries to search for
        :type query: dict
        :rtype: :class:`mopidy.models.Playlist`
        """
        return self.provider.search(**query)


class BaseLibraryProvider(object):
    """
    :param backend: backend the controller is a part of
    :type backend: :class:`mopidy.backends.base.Backend`
    """

    pykka_traversable = True

    def __init__(self, backend):
        self.backend = backend

    def find_exact(self, **query):
        """
        See :meth:`mopidy.backends.base.LibraryController.find_exact`.

        *MUST be implemented by subclass.*
        """
        raise NotImplementedError

    def lookup(self, uri):
        """
        See :meth:`mopidy.backends.base.LibraryController.lookup`.

        *MUST be implemented by subclass.*
        """
        raise NotImplementedError

    def refresh(self, uri=None):
        """
        See :meth:`mopidy.backends.base.LibraryController.refresh`.

        *MUST be implemented by subclass.*
        """
        raise NotImplementedError

    def search(self, **query):
        """
        See :meth:`mopidy.backends.base.LibraryController.search`.

        *MUST be implemented by subclass.*
        """
        raise NotImplementedError



class BaseMixerTest(object):
    MIN = 0
    MAX = 100
    ACTUAL_MIN = MIN
    ACTUAL_MAX = MAX
    INITIAL = None

    mixer_class = None

    def setUp(self):
        assert self.mixer_class is not None, \
            "mixer_class must be set in subclass"
        # pylint: disable = E1102
        self.mixer = self.mixer_class()
        # pylint: enable = E1102

    def test_initial_volume(self):
        self.assertEqual(self.mixer.volume, self.INITIAL)

    def test_volume_set_to_min(self):
        self.mixer.volume = self.MIN
        self.assertEqual(self.mixer.volume, self.ACTUAL_MIN)

    def test_volume_set_to_max(self):
        self.mixer.volume = self.MAX
        self.assertEqual(self.mixer.volume, self.ACTUAL_MAX)

    def test_volume_set_to_below_min_results_in_min(self):
        self.mixer.volume = -10
        self.assertEqual(self.mixer.volume, self.ACTUAL_MIN)

    def test_volume_set_to_above_max_results_in_max(self):
        self.mixer.volume = self.MAX + 10
        self.assertEqual(self.mixer.volume, self.ACTUAL_MAX)

    def test_volume_is_not_float(self):
        self.mixer.volume = 1.0 / 3 * 100
        self.assertEqual(self.mixer.volume, 33)



from mopidy import settings

from tests.frontends.mpd import protocol


class ReflectionHandlerTest(protocol.BaseTestCase):
    def test_commands_returns_list_of_all_commands(self):
        self.sendRequest(u'commands')
        # Check if some random commands are included
        self.assertInResponse(u'command: commands')
        self.assertInResponse(u'command: play')
        self.assertInResponse(u'command: status')
        # Check if commands you do not have access to are not present
        self.assertNotInResponse(u'command: kill')
        # Check if the blacklisted commands are not present
        self.assertNotInResponse(u'command: command_list_begin')
        self.assertNotInResponse(u'command: command_list_ok_begin')
        self.assertNotInResponse(u'command: command_list_end')
        self.assertNotInResponse(u'command: idle')
        self.assertNotInResponse(u'command: noidle')
        self.assertNotInResponse(u'command: sticker')
        self.assertInResponse(u'OK')

    def test_commands_show_less_if_auth_required_and_not_authed(self):
        settings.MPD_SERVER_PASSWORD = u'secret'
        self.sendRequest(u'commands')
        # Not requiring auth
        self.assertInResponse(u'command: close')
        self.assertInResponse(u'command: commands')
        self.assertInResponse(u'command: notcommands')
        self.assertInResponse(u'command: password')
        self.assertInResponse(u'command: ping')
        # Requiring auth
        self.assertNotInResponse(u'command: play')
        self.assertNotInResponse(u'command: status')

    def test_decoders(self):
        self.sendRequest(u'decoders')
        self.assertInResponse(u'ACK [0@0] {} Not implemented')

    def test_notcommands_returns_only_kill_and_ok(self):
        response = self.sendRequest(u'notcommands')
        self.assertEqual(2, len(response))
        self.assertInResponse(u'command: kill')
        self.assertInResponse(u'OK')

    def test_notcommands_returns_more_if_auth_required_and_not_authed(self):
        settings.MPD_SERVER_PASSWORD = u'secret'
        self.sendRequest(u'notcommands')
        # Not requiring auth
        self.assertNotInResponse(u'command: close')
        self.assertNotInResponse(u'command: commands')
        self.assertNotInResponse(u'command: notcommands')
        self.assertNotInResponse(u'command: password')
        self.assertNotInResponse(u'command: ping')
        # Requiring auth
        self.assertInResponse(u'command: play')
        self.assertInResponse(u'command: status')

    def test_tagtypes(self):
        self.sendRequest(u'tagtypes')
        self.assertInResponse(u'OK')

    def test_urlhandlers(self):
        self.sendRequest(u'urlhandlers')
        self.assertInResponse(u'OK')
        self.assertInResponse(u'handler: dummy')



import re
import shlex

from mopidy.frontends.mpd.exceptions import MpdArgError, MpdNotImplemented
from mopidy.frontends.mpd.protocol import handle_request, stored_playlists
from mopidy.frontends.mpd.translator import playlist_to_mpd_format

def _build_query(mpd_query):
    """
    Parses a MPD query string and converts it to the Mopidy query format.
    """
    query_pattern = (
        r'"?(?:[Aa]lbum|[Aa]rtist|[Ff]ilename|[Tt]itle|[Aa]ny)"? "[^"]+"')
    query_parts = re.findall(query_pattern, mpd_query)
    query_part_pattern = (
        r'"?(?P<field>([Aa]lbum|[Aa]rtist|[Ff]ilename|[Tt]itle|[Aa]ny))"? '
        r'"(?P<what>[^"]+)"')
    query = {}
    for query_part in query_parts:
        m = re.match(query_part_pattern, query_part)
        field = m.groupdict()['field'].lower()
        if field == u'title':
            field = u'track'
        field = str(field) # Needed for kwargs keys on OS X and Windows
        what = m.groupdict()['what'].lower()
        if field in query:
            query[field].append(what)
        else:
            query[field] = [what]
    return query

@handle_request(r'^count "(?P<tag>[^"]+)" "(?P<needle>[^"]*)"$')
def count(context, tag, needle):
    """
    *musicpd.org, music database section:*

        ``count {TAG} {NEEDLE}``

        Counts the number of songs and their total playtime in the db
        matching ``TAG`` exactly.
    """
    return [('songs', 0), ('playtime', 0)] # TODO

@handle_request(r'^find '
     r'(?P<mpd_query>("?([Aa]lbum|[Aa]rtist|[Dd]ate|[Ff]ilename|'
     r'[Tt]itle|[Aa]ny)"? "[^"]+"\s?)+)$')
def find(context, mpd_query):
    """
    *musicpd.org, music database section:*

        ``find {TYPE} {WHAT}``

        Finds songs in the db that are exactly ``WHAT``. ``TYPE`` should be
        ``album``, ``artist``, or ``title``. ``WHAT`` is what to find.

    *GMPC:*

    - does not add quotes around the field argument.
    - also uses ``find album "[ALBUM]" artist "[ARTIST]"`` to list album
      tracks.

    *ncmpc:*

    - does not add quotes around the field argument.
    - capitalizes the type argument.

    *ncmpcpp:*

    - also uses the search type "date".
    """
    query = _build_query(mpd_query)
    return playlist_to_mpd_format(
        context.backend.library.find_exact(**query).get())

@handle_request(r'^findadd '
     r'(?P<query>("?([Aa]lbum|[Aa]rtist|[Ff]ilename|[Tt]itle|[Aa]ny)"? '
     '"[^"]+"\s?)+)$')
def findadd(context, query):
    """
    *musicpd.org, music database section:*

        ``findadd {TYPE} {WHAT}``

        Finds songs in the db that are exactly ``WHAT`` and adds them to
        current playlist. ``TYPE`` can be any tag supported by MPD.
        ``WHAT`` is what to find.
    """
    # TODO Add result to current playlist
    #result = context.find(query)

@handle_request(r'^list "?(?P<field>([Aa]rtist|[Aa]lbum|[Dd]ate|[Gg]enre))"?'
    '( (?P<mpd_query>.*))?$')
def list_(context, field, mpd_query=None):
    """
    *musicpd.org, music database section:*

        ``list {TYPE} [ARTIST]``

        Lists all tags of the specified type. ``TYPE`` should be ``album``,
        ``artist``, ``date``, or ``genre``.

        ``ARTIST`` is an optional parameter when type is ``album``,
        ``date``, or ``genre``.

        This filters the result list by an artist.

    *Clarifications:*

        The musicpd.org documentation for ``list`` is far from complete. The
        command also supports the following variant:

        ``list {TYPE} {QUERY}``

        Where ``QUERY`` applies to all ``TYPE``. ``QUERY`` is one or more pairs
        of a field name and a value. If the ``QUERY`` consists of more than one
        pair, the pairs are AND-ed together to find the result. Examples of
        valid queries and what they should return:

        ``list "artist" "artist" "ABBA"``
            List artists where the artist name is "ABBA". Response::

                Artist: ABBA
                OK

        ``list "album" "artist" "ABBA"``
            Lists albums where the artist name is "ABBA". Response::

                Album: More ABBA Gold: More ABBA Hits
                Album: Absolute More Christmas
                Album: Gold: Greatest Hits
                OK

        ``list "artist" "album" "Gold: Greatest Hits"``
            Lists artists where the album name is "Gold: Greatest Hits".
            Response::

                Artist: ABBA
                OK

        ``list "artist" "artist" "ABBA" "artist" "TLC"``
            Lists artists where the artist name is "ABBA" *and* "TLC". Should
            never match anything. Response::

                OK

        ``list "date" "artist" "ABBA"``
            Lists dates where artist name is "ABBA". Response::

                Date:
                Date: 1992
                Date: 1993
                OK

        ``list "date" "artist" "ABBA" "album" "Gold: Greatest Hits"``
            Lists dates where artist name is "ABBA" and album name is "Gold:
            Greatest Hits". Response::

                Date: 1992
                OK

        ``list "genre" "artist" "The Rolling Stones"``
            Lists genres where artist name is "The Rolling Stones". Response::

                Genre:
                Genre: Rock
                OK

    *GMPC:*

    - does not add quotes around the field argument.

    *ncmpc:*

    - does not add quotes around the field argument.
    - capitalizes the field argument.
    """
    field = field.lower()
    query = _list_build_query(field, mpd_query)
    if field == u'artist':
        return _list_artist(context, query)
    elif field == u'album':
        return _list_album(context, query)
    elif field == u'date':
        return _list_date(context, query)
    elif field == u'genre':
        pass # TODO We don't have genre in our internal data structures yet

def _list_build_query(field, mpd_query):
    """Converts a ``list`` query to a Mopidy query."""
    if mpd_query is None:
        return {}
    try:
        # shlex does not seem to be friends with unicode objects
        tokens = shlex.split(mpd_query.encode('utf-8'))
    except ValueError as error:
        if error.message == 'No closing quotation':
            raise MpdArgError(u'Invalid unquoted character', command=u'list')
        else:
            raise error
    tokens = [t.decode('utf-8') for t in tokens]
    if len(tokens) == 1:
        if field == u'album':
            return {'artist': [tokens[0]]}
        else:
            raise MpdArgError(
                u'should be "Album" for 3 arguments', command=u'list')
    elif len(tokens) % 2 == 0:
        query = {}
        while tokens:
            key = tokens[0].lower()
            key = str(key) # Needed for kwargs keys on OS X and Windows
            value = tokens[1]
            tokens = tokens[2:]
            if key not in (u'artist', u'album', u'date', u'genre'):
                raise MpdArgError(u'not able to parse args', command=u'list')
            if key in query:
                query[key].append(value)
            else:
                query[key] = [value]
        return query
    else:
        raise MpdArgError(u'not able to parse args', command=u'list')

def _list_artist(context, query):
    artists = set()
    playlist = context.backend.library.find_exact(**query).get()
    for track in playlist.tracks:
        for artist in track.artists:
            artists.add((u'Artist', artist.name))
    return artists

def _list_album(context, query):
    albums = set()
    playlist = context.backend.library.find_exact(**query).get()
    for track in playlist.tracks:
        if track.album is not None:
            albums.add((u'Album', track.album.name))
    return albums

def _list_date(context, query):
    dates = set()
    playlist = context.backend.library.find_exact(**query).get()
    for track in playlist.tracks:
        if track.date is not None:
            dates.add((u'Date', track.date.strftime('%Y-%m-%d')))
    return dates

@handle_request(r'^listall "(?P<uri>[^"]+)"')
def listall(context, uri):
    """
    *musicpd.org, music database section:*

        ``listall [URI]``

        Lists all songs and directories in ``URI``.
    """
    raise MpdNotImplemented # TODO

@handle_request(r'^listallinfo "(?P<uri>[^"]+)"')
def listallinfo(context, uri):
    """
    *musicpd.org, music database section:*

        ``listallinfo [URI]``

        Same as ``listall``, except it also returns metadata info in the
        same format as ``lsinfo``.
    """
    raise MpdNotImplemented # TODO

@handle_request(r'^lsinfo$')
@handle_request(r'^lsinfo "(?P<uri>[^"]*)"$')
def lsinfo(context, uri=None):
    """
    *musicpd.org, music database section:*

        ``lsinfo [URI]``

        Lists the contents of the directory ``URI``.

        When listing the root directory, this currently returns the list of
        stored playlists. This behavior is deprecated; use
        ``listplaylists`` instead.

    MPD returns the same result, including both playlists and the files and
    directories located at the root level, for both ``lsinfo``, ``lsinfo
    ""``, and ``lsinfo "/"``.
    """
    if uri is None or uri == u'/' or uri == u'':
        return stored_playlists.listplaylists(context)
    raise MpdNotImplemented # TODO

@handle_request(r'^rescan( "(?P<uri>[^"]+)")*$')
def rescan(context, uri=None):
    """
    *musicpd.org, music database section:*

        ``rescan [URI]``

        Same as ``update``, but also rescans unmodified files.
    """
    return update(context, uri, rescan_unmodified_files=True)

@handle_request(r'^search '
     r'(?P<mpd_query>("?([Aa]lbum|[Aa]rtist|[Dd]ate|[Ff]ilename|'
     r'[Tt]itle|[Aa]ny)"? "[^"]+"\s?)+)$')
def search(context, mpd_query):
    """
    *musicpd.org, music database section:*

        ``search {TYPE} {WHAT}``

        Searches for any song that contains ``WHAT``. ``TYPE`` can be
        ``title``, ``artist``, ``album`` or ``filename``. Search is not
        case sensitive.

    *GMPC:*

    - does not add quotes around the field argument.
    - uses the undocumented field ``any``.
    - searches for multiple words like this::

        search any "foo" any "bar" any "baz"

    *ncmpc:*

    - does not add quotes around the field argument.
    - capitalizes the field argument.

    *ncmpcpp:*

    - also uses the search type "date".
    """
    query = _build_query(mpd_query)
    return playlist_to_mpd_format(
        context.backend.library.search(**query).get())

@handle_request(r'^update( "(?P<uri>[^"]+)")*$')
def update(context, uri=None, rescan_unmodified_files=False):
    """
    *musicpd.org, music database section:*

        ``update [URI]``

        Updates the music database: find new files, remove deleted files,
        update modified files.

        ``URI`` is a particular directory or song/file to update. If you do
        not specify it, everything is updated.

        Prints ``updating_db: JOBID`` where ``JOBID`` is a positive number
        identifying the update job. You can read the current job id in the
        ``status`` response.
    """
    return {'updating_db': 0} # TODO



from mopidy.frontends.mpd.protocol import handle_request, mpd_commands
from mopidy.frontends.mpd.exceptions import MpdNotImplemented

@handle_request(r'^commands$', auth_required=False)
def commands(context):
    """
    *musicpd.org, reflection section:*

        ``commands``

        Shows which commands the current user has access to.
    """
    if context.dispatcher.authenticated:
        command_names = set([command.name for command in mpd_commands])
    else:
        command_names = set([command.name for command in mpd_commands
            if not command.auth_required])

    # No one is permited to use kill, rest of commands are not listed by MPD,
    # so we shouldn't either.
    command_names = command_names - set(['kill', 'command_list_begin',
        'command_list_ok_begin', 'command_list_ok_begin', 'command_list_end',
        'idle', 'noidle', 'sticker'])

    return [('command', command_name) for command_name in sorted(command_names)]

@handle_request(r'^decoders$')
def decoders(context):
    """
    *musicpd.org, reflection section:*

        ``decoders``

        Print a list of decoder plugins, followed by their supported
        suffixes and MIME types. Example response::

            plugin: mad
            suffix: mp3
            suffix: mp2
            mime_type: audio/mpeg
            plugin: mpcdec
            suffix: mpc
    """
    raise MpdNotImplemented # TODO

@handle_request(r'^notcommands$', auth_required=False)
def notcommands(context):
    """
    *musicpd.org, reflection section:*

        ``notcommands``

        Shows which commands the current user does not have access to.
    """
    if context.dispatcher.authenticated:
        command_names = []
    else:
        command_names = [command.name for command in mpd_commands
            if command.auth_required]

    # No permission to use
    command_names.append('kill')

    return [('command', command_name) for command_name in sorted(command_names)]

@handle_request(r'^tagtypes$')
def tagtypes(context):
    """
    *musicpd.org, reflection section:*

        ``tagtypes``

        Shows a list of available song metadata.
    """
    pass # TODO

@handle_request(r'^urlhandlers$')
def urlhandlers(context):
    """
    *musicpd.org, reflection section:*

        ``urlhandlers``

        Gets a list of available URL handlers.
    """
    return [(u'handler', uri_scheme)
        for uri_scheme in context.backend.uri_schemes.get()]



import logging

from mopidy import settings
from mopidy.outputs import BaseOutput

logger = logging.getLogger('mopidy.outputs.shoutcast')

class ShoutcastOutput(BaseOutput):
    """
    Shoutcast streaming output.

    This output allows for streaming to an icecast server or anything else that
    supports Shoutcast. The output supports setting for: server address, port,
    mount point, user, password and encoder to use. Please see
    :class:`mopidy.settings` for details about settings.

    **Dependencies:**

    - A SHOUTcast/Icecast server

    **Settings:**

    - :attr:`mopidy.settings.SHOUTCAST_OUTPUT_HOSTNAME`
    - :attr:`mopidy.settings.SHOUTCAST_OUTPUT_PORT`
    - :attr:`mopidy.settings.SHOUTCAST_OUTPUT_USERNAME`
    - :attr:`mopidy.settings.SHOUTCAST_OUTPUT_PASSWORD`
    - :attr:`mopidy.settings.SHOUTCAST_OUTPUT_MOUNT`
    - :attr:`mopidy.settings.SHOUTCAST_OUTPUT_ENCODER`
    """

    def describe_bin(self):
        return 'audioconvert ! %s ! shout2send name=shoutcast' \
            % settings.SHOUTCAST_OUTPUT_ENCODER

    def modify_bin(self):
        self.set_properties(self.bin.get_by_name('shoutcast'), {
            u'ip': settings.SHOUTCAST_OUTPUT_HOSTNAME,
            u'port': settings.SHOUTCAST_OUTPUT_PORT,
            u'mount': settings.SHOUTCAST_OUTPUT_MOUNT,
            u'username': settings.SHOUTCAST_OUTPUT_USERNAME,
            u'password': settings.SHOUTCAST_OUTPUT_PASSWORD,
        })

    def on_connect(self):
        self.gstreamer.connect_message_handler(
            self.bin.get_by_name('shoutcast'), self.message_handler)

    def on_remove(self):
        self.gstreamer.remove_message_handler(
            self.bin.get_by_name('shoutcast'))

    def message_handler(self, message):
        if message.type != self.MESSAGE_ERROR:
            return False
        error, debug = message.parse_error()
        logger.warning('%s (%s)', error, debug)
        self.remove()
        return True



from mopidy.backends.base import BaseStoredPlaylistsProvider

class SpotifyStoredPlaylistsProvider(BaseStoredPlaylistsProvider):
    def create(self, name):
        pass # TODO

    def delete(self, playlist):
        pass # TODO

    def lookup(self, uri):
        pass # TODO

    def refresh(self):
        pass # TODO

    def rename(self, playlist, new_name):
        pass # TODO

    def save(self, playlist):
        pass # TODO



import logging
import random
import time

from mopidy.listeners import BackendListener

logger = logging.getLogger('mopidy.backends.base')


def option_wrapper(name, default):
    def get_option(self):
        return getattr(self, name, default)
    def set_option(self, value):
        if getattr(self, name, default) != value:
            self._trigger_options_changed()
        return setattr(self, name, value)
    return property(get_option, set_option)


class PlaybackController(object):
    """
    :param backend: the backend
    :type backend: :class:`mopidy.backends.base.Backend`
    :param provider: provider the controller should use
    :type provider: instance of :class:`BasePlaybackProvider`
    """

    # pylint: disable = R0902
    # Too many instance attributes

    pykka_traversable = True

    #: Constant representing the paused state.
    PAUSED = u'paused'

    #: Constant representing the playing state.
    PLAYING = u'playing'

    #: Constant representing the stopped state.
    STOPPED = u'stopped'

    #: :class:`True`
    #:     Tracks are removed from the playlist when they have been played.
    #: :class:`False`
    #:     Tracks are not removed from the playlist.
    consume = option_wrapper('_consume', False)

    #: The currently playing or selected track.
    #:
    #: A two-tuple of (CPID integer, :class:`mopidy.models.Track`) or
    #: :class:`None`.
    current_cp_track = None

    #: :class:`True`
    #:     Tracks are selected at random from the playlist.
    #: :class:`False`
    #:     Tracks are played in the order of the playlist.
    random = option_wrapper('_random', False)

    #: :class:`True`
    #:     The current playlist is played repeatedly. To repeat a single track,
    #:     select both :attr:`repeat` and :attr:`single`.
    #: :class:`False`
    #:     The current playlist is played once.
    repeat = option_wrapper('_repeat', False)

    #: :class:`True`
    #:     Playback is stopped after current song, unless in :attr:`repeat`
    #:     mode.
    #: :class:`False`
    #:     Playback continues after current song.
    single = option_wrapper('_single', False)

    def __init__(self, backend, provider):
        self.backend = backend
        self.provider = provider
        self._state = self.STOPPED
        self._shuffled = []
        self._first_shuffle = True
        self.play_time_accumulated = 0
        self.play_time_started = None

    def _get_cpid(self, cp_track):
        if cp_track is None:
            return None
        return cp_track.cpid

    def _get_track(self, cp_track):
        if cp_track is None:
            return None
        return cp_track.track

    @property
    def current_cpid(self):
        """
        The CPID (current playlist ID) of the currently playing or selected
        track.

        Read-only. Extracted from :attr:`current_cp_track` for convenience.
        """
        return self._get_cpid(self.current_cp_track)

    @property
    def current_track(self):
        """
        The currently playing or selected :class:`mopidy.models.Track`.

        Read-only. Extracted from :attr:`current_cp_track` for convenience.
        """
        return self._get_track(self.current_cp_track)

    @property
    def current_playlist_position(self):
        """
        The position of the current track in the current playlist.

        Read-only.
        """
        if self.current_cp_track is None:
            return None
        try:
            return self.backend.current_playlist.cp_tracks.index(
                self.current_cp_track)
        except ValueError:
            return None

    @property
    def track_at_eot(self):
        """
        The track that will be played at the end of the current track.

        Read-only. A :class:`mopidy.models.Track` extracted from
        :attr:`cp_track_at_eot` for convenience.
        """
        return self._get_track(self.cp_track_at_eot)

    @property
    def cp_track_at_eot(self):
        """
        The track that will be played at the end of the current track.

        Read-only. A two-tuple of (CPID integer, :class:`mopidy.models.Track`).

        Not necessarily the same track as :attr:`cp_track_at_next`.
        """
        # pylint: disable = R0911
        # Too many return statements

        cp_tracks = self.backend.current_playlist.cp_tracks

        if not cp_tracks:
            return None

        if self.random and not self._shuffled:
            if self.repeat or self._first_shuffle:
                logger.debug('Shuffling tracks')
                self._shuffled = cp_tracks
                random.shuffle(self._shuffled)
                self._first_shuffle = False

        if self.random and self._shuffled:
            return self._shuffled[0]

        if self.current_cp_track is None:
            return cp_tracks[0]

        if self.repeat and self.single:
            return cp_tracks[self.current_playlist_position]

        if self.repeat and not self.single:
            return cp_tracks[
                (self.current_playlist_position + 1) % len(cp_tracks)]

        try:
            return cp_tracks[self.current_playlist_position + 1]
        except IndexError:
            return None

    @property
    def track_at_next(self):
        """
        The track that will be played if calling :meth:`next()`.

        Read-only. A :class:`mopidy.models.Track` extracted from
        :attr:`cp_track_at_next` for convenience.
        """
        return self._get_track(self.cp_track_at_next)

    @property
    def cp_track_at_next(self):
        """
        The track that will be played if calling :meth:`next()`.

        Read-only. A two-tuple of (CPID integer, :class:`mopidy.models.Track`).

        For normal playback this is the next track in the playlist. If repeat
        is enabled the next track can loop around the playlist. When random is
        enabled this should be a random track, all tracks should be played once
        before the list repeats.
        """
        cp_tracks = self.backend.current_playlist.cp_tracks

        if not cp_tracks:
            return None

        if self.random and not self._shuffled:
            if self.repeat or self._first_shuffle:
                logger.debug('Shuffling tracks')
                self._shuffled = cp_tracks
                random.shuffle(self._shuffled)
                self._first_shuffle = False

        if self.random and self._shuffled:
            return self._shuffled[0]

        if self.current_cp_track is None:
            return cp_tracks[0]

        if self.repeat:
            return cp_tracks[
                (self.current_playlist_position + 1) % len(cp_tracks)]

        try:
            return cp_tracks[self.current_playlist_position + 1]
        except IndexError:
            return None

    @property
    def track_at_previous(self):
        """
        The track that will be played if calling :meth:`previous()`.

        Read-only. A :class:`mopidy.models.Track` extracted from
        :attr:`cp_track_at_previous` for convenience.
        """
        return self._get_track(self.cp_track_at_previous)

    @property
    def cp_track_at_previous(self):
        """
        The track that will be played if calling :meth:`previous()`.

        A two-tuple of (CPID integer, :class:`mopidy.models.Track`).

        For normal playback this is the previous track in the playlist. If
        random and/or consume is enabled it should return the current track
        instead.
        """
        if self.repeat or self.consume or self.random:
            return self.current_cp_track

        if self.current_playlist_position in (None, 0):
            return None

        return self.backend.current_playlist.cp_tracks[
            self.current_playlist_position - 1]

    @property
    def state(self):
        """
        The playback state. Must be :attr:`PLAYING`, :attr:`PAUSED`, or
        :attr:`STOPPED`.

        Possible states and transitions:

        .. digraph:: state_transitions

            "STOPPED" -> "PLAYING" [ label="play" ]
            "STOPPED" -> "PAUSED" [ label="pause" ]
            "PLAYING" -> "STOPPED" [ label="stop" ]
            "PLAYING" -> "PAUSED" [ label="pause" ]
            "PLAYING" -> "PLAYING" [ label="play" ]
            "PAUSED" -> "PLAYING" [ label="resume" ]
            "PAUSED" -> "STOPPED" [ label="stop" ]
        """
        return self._state

    @state.setter
    def state(self, new_state):
        (old_state, self._state) = (self.state, new_state)
        logger.debug(u'Changing state: %s -> %s', old_state, new_state)

        self._trigger_playback_state_changed()

        # FIXME play_time stuff assumes backend does not have a better way of
        # handeling this stuff :/
        if (old_state in (self.PLAYING, self.STOPPED)
                and new_state == self.PLAYING):
            self._play_time_start()
        elif old_state == self.PLAYING and new_state == self.PAUSED:
            self._play_time_pause()
        elif old_state == self.PAUSED and new_state == self.PLAYING:
            self._play_time_resume()

    @property
    def time_position(self):
        """Time position in milliseconds."""
        if self.state == self.PLAYING:
            time_since_started = (self._current_wall_time -
                self.play_time_started)
            return self.play_time_accumulated + time_since_started
        elif self.state == self.PAUSED:
            return self.play_time_accumulated
        elif self.state == self.STOPPED:
            return 0

    def _play_time_start(self):
        self.play_time_accumulated = 0
        self.play_time_started = self._current_wall_time

    def _play_time_pause(self):
        time_since_started = self._current_wall_time - self.play_time_started
        self.play_time_accumulated += time_since_started

    def _play_time_resume(self):
        self.play_time_started = self._current_wall_time

    @property
    def _current_wall_time(self):
        return int(time.time() * 1000)

    def change_track(self, cp_track, on_error_step=1):
        """
        Change to the given track, keeping the current playback state.

        :param cp_track: track to change to
        :type cp_track: two-tuple (CPID integer, :class:`mopidy.models.Track`)
            or :class:`None`
        :param on_error_step: direction to step at play error, 1 for next
            track (default), -1 for previous track
        :type on_error_step: int, -1 or 1

        """
        old_state = self.state
        self.stop()
        self.current_cp_track = cp_track
        if old_state == self.PLAYING:
            self.play(on_error_step=on_error_step)
        elif old_state == self.PAUSED:
            self.pause()

    def on_end_of_track(self):
        """
        Tell the playback controller that end of track is reached.

        Typically called by :class:`mopidy.process.CoreProcess` after a message
        from a library thread is received.
        """
        if self.state == self.STOPPED:
            return

        original_cp_track = self.current_cp_track

        if self.cp_track_at_eot:
            self._trigger_track_playback_ended()
            self.play(self.cp_track_at_eot)
        else:
            self.stop(clear_current_track=True)

        if self.consume:
            self.backend.current_playlist.remove(cpid=original_cp_track.cpid)

    def on_current_playlist_change(self):
        """
        Tell the playback controller that the current playlist has changed.

        Used by :class:`mopidy.backends.base.CurrentPlaylistController`.
        """
        self._first_shuffle = True
        self._shuffled = []

        if (not self.backend.current_playlist.cp_tracks or
                self.current_cp_track not in
                self.backend.current_playlist.cp_tracks):
            self.stop(clear_current_track=True)

    def next(self):
        """
        Change to the next track.

        The current playback state will be kept. If it was playing, playing
        will continue. If it was paused, it will still be paused, etc.
        """
        if self.cp_track_at_next:
            self._trigger_track_playback_ended()
            self.change_track(self.cp_track_at_next)
        else:
            self.stop(clear_current_track=True)

    def pause(self):
        """Pause playback."""
        if self.provider.pause():
            self.state = self.PAUSED
            self._trigger_track_playback_paused()

    def play(self, cp_track=None, on_error_step=1):
        """
        Play the given track, or if the given track is :class:`None`, play the
        currently active track.

        :param cp_track: track to play
        :type cp_track: two-tuple (CPID integer, :class:`mopidy.models.Track`)
            or :class:`None`
        :param on_error_step: direction to step at play error, 1 for next
            track (default), -1 for previous track
        :type on_error_step: int, -1 or 1
        """

        if cp_track is not None:
            assert cp_track in self.backend.current_playlist.cp_tracks
        elif cp_track is None:
            if self.state == self.PAUSED:
                return self.resume()
            elif self.current_cp_track is not None:
                cp_track = self.current_cp_track
            elif self.current_cp_track is None and on_error_step == 1:
                cp_track = self.cp_track_at_next
            elif self.current_cp_track is None and on_error_step == -1:
                cp_track = self.cp_track_at_previous

        if cp_track is not None:
            self.current_cp_track = cp_track
            self.state = self.PLAYING
            if not self.provider.play(cp_track.track):
                # Track is not playable
                if self.random and self._shuffled:
                    self._shuffled.remove(cp_track)
                if on_error_step == 1:
                    self.next()
                elif on_error_step == -1:
                    self.previous()

        if self.random and self.current_cp_track in self._shuffled:
            self._shuffled.remove(self.current_cp_track)

        self._trigger_track_playback_started()

    def previous(self):
        """
        Change to the previous track.

        The current playback state will be kept. If it was playing, playing
        will continue. If it was paused, it will still be paused, etc.
        """
        self._trigger_track_playback_ended()
        self.change_track(self.cp_track_at_previous, on_error_step=-1)

    def resume(self):
        """If paused, resume playing the current track."""
        if self.state == self.PAUSED and self.provider.resume():
            self.state = self.PLAYING
            self._trigger_track_playback_resumed()

    def seek(self, time_position):
        """
        Seeks to time position given in milliseconds.

        :param time_position: time position in milliseconds
        :type time_position: int
        :rtype: :class:`True` if successful, else :class:`False`
        """
        if not self.backend.current_playlist.tracks:
            return False

        if self.state == self.STOPPED:
            self.play()
        elif self.state == self.PAUSED:
            self.resume()

        if time_position < 0:
            time_position = 0
        elif time_position > self.current_track.length:
            self.next()
            return True

        self.play_time_started = self._current_wall_time
        self.play_time_accumulated = time_position

        success = self.provider.seek(time_position)
        if success:
            self._trigger_seeked()
        return success

    def stop(self, clear_current_track=False):
        """
        Stop playing.

        :param clear_current_track: whether to clear the current track _after_
            stopping
        :type clear_current_track: boolean
        """
        if self.state != self.STOPPED:
            if self.provider.stop():
                self._trigger_track_playback_ended()
                self.state = self.STOPPED
        if clear_current_track:
            self.current_cp_track = None

    def _trigger_track_playback_paused(self):
        logger.debug(u'Triggering track playback paused event')
        if self.current_track is None:
            return
        BackendListener.send('track_playback_paused',
            track=self.current_track,
            time_position=self.time_position)

    def _trigger_track_playback_resumed(self):
        logger.debug(u'Triggering track playback resumed event')
        if self.current_track is None:
            return
        BackendListener.send('track_playback_resumed',
            track=self.current_track,
            time_position=self.time_position)

    def _trigger_track_playback_started(self):
        logger.debug(u'Triggering track playback started event')
        if self.current_track is None:
            return
        BackendListener.send('track_playback_started',
            track=self.current_track)

    def _trigger_track_playback_ended(self):
        logger.debug(u'Triggering track playback ended event')
        if self.current_track is None:
            return
        BackendListener.send('track_playback_ended',
            track=self.current_track,
            time_position=self.time_position)

    def _trigger_playback_state_changed(self):
        logger.debug(u'Triggering playback state change event')
        BackendListener.send('playback_state_changed')

    def _trigger_options_changed(self):
        logger.debug(u'Triggering options changed event')
        BackendListener.send('options_changed')

    def _trigger_seeked(self):
        logger.debug(u'Triggering seeked event')
        BackendListener.send('seeked')


class BasePlaybackProvider(object):
    """
    :param backend: the backend
    :type backend: :class:`mopidy.backends.base.Backend`
    """

    pykka_traversable = True

    def __init__(self, backend):
        self.backend = backend

    def pause(self):
        """
        Pause playback.

        *MUST be implemented by subclass.*

        :rtype: :class:`True` if successful, else :class:`False`
        """
        raise NotImplementedError

    def play(self, track):
        """
        Play given track.

        *MUST be implemented by subclass.*

        :param track: the track to play
        :type track: :class:`mopidy.models.Track`
        :rtype: :class:`True` if successful, else :class:`False`
        """
        raise NotImplementedError

    def resume(self):
        """
        Resume playback at the same time position playback was paused.

        *MUST be implemented by subclass.*

        :rtype: :class:`True` if successful, else :class:`False`
        """
        raise NotImplementedError

    def seek(self, time_position):
        """
        Seek to a given time position.

        *MUST be implemented by subclass.*

        :param time_position: time position in milliseconds
        :type time_position: int
        :rtype: :class:`True` if successful, else :class:`False`
        """
        raise NotImplementedError

    def stop(self):
        """
        Stop playback.

        *MUST be implemented by subclass.*

        :rtype: :class:`True` if successful, else :class:`False`
        """
        raise NotImplementedError



from tests.frontends.mpd import protocol


class MusicDatabaseHandlerTest(protocol.BaseTestCase):
    def test_count(self):
        self.sendRequest(u'count "tag" "needle"')
        self.assertInResponse(u'songs: 0')
        self.assertInResponse(u'playtime: 0')
        self.assertInResponse(u'OK')

    def test_findadd(self):
        self.sendRequest(u'findadd "album" "what"')
        self.assertInResponse(u'OK')

    def test_listall(self):
        self.sendRequest(u'listall "file:///dev/urandom"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_listallinfo(self):
        self.sendRequest(u'listallinfo "file:///dev/urandom"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_lsinfo_without_path_returns_same_as_listplaylists(self):
        lsinfo_response = self.sendRequest(u'lsinfo')
        listplaylists_response = self.sendRequest(u'listplaylists')
        self.assertEqual(lsinfo_response, listplaylists_response)

    def test_lsinfo_with_empty_path_returns_same_as_listplaylists(self):
        lsinfo_response = self.sendRequest(u'lsinfo ""')
        listplaylists_response = self.sendRequest(u'listplaylists')
        self.assertEqual(lsinfo_response, listplaylists_response)

    def test_lsinfo_for_root_returns_same_as_listplaylists(self):
        lsinfo_response = self.sendRequest(u'lsinfo "/"')
        listplaylists_response = self.sendRequest(u'listplaylists')
        self.assertEqual(lsinfo_response, listplaylists_response)

    def test_update_without_uri(self):
        self.sendRequest(u'update')
        self.assertInResponse(u'updating_db: 0')
        self.assertInResponse(u'OK')

    def test_update_with_uri(self):
        self.sendRequest(u'update "file:///dev/urandom"')
        self.assertInResponse(u'updating_db: 0')
        self.assertInResponse(u'OK')

    def test_rescan_without_uri(self):
        self.sendRequest(u'rescan')
        self.assertInResponse(u'updating_db: 0')
        self.assertInResponse(u'OK')

    def test_rescan_with_uri(self):
        self.sendRequest(u'rescan "file:///dev/urandom"')
        self.assertInResponse(u'updating_db: 0')
        self.assertInResponse(u'OK')


class MusicDatabaseFindTest(protocol.BaseTestCase):
    def test_find_album(self):
        self.sendRequest(u'find "album" "what"')
        self.assertInResponse(u'OK')

    def test_find_album_without_quotes(self):
        self.sendRequest(u'find album "what"')
        self.assertInResponse(u'OK')

    def test_find_artist(self):
        self.sendRequest(u'find "artist" "what"')
        self.assertInResponse(u'OK')

    def test_find_artist_without_quotes(self):
        self.sendRequest(u'find artist "what"')
        self.assertInResponse(u'OK')

    def test_find_title(self):
        self.sendRequest(u'find "title" "what"')
        self.assertInResponse(u'OK')

    def test_find_title_without_quotes(self):
        self.sendRequest(u'find title "what"')
        self.assertInResponse(u'OK')

    def test_find_date(self):
        self.sendRequest(u'find "date" "2002-01-01"')
        self.assertInResponse(u'OK')

    def test_find_date_without_quotes(self):
        self.sendRequest(u'find date "2002-01-01"')
        self.assertInResponse(u'OK')

    def test_find_date_with_capital_d_and_incomplete_date(self):
        self.sendRequest(u'find Date "2005"')
        self.assertInResponse(u'OK')

    def test_find_else_should_fail(self):
        self.sendRequest(u'find "somethingelse" "what"')
        self.assertEqualResponse(u'ACK [2@0] {find} incorrect arguments')

    def test_find_album_and_artist(self):
        self.sendRequest(u'find album "album_what" artist "artist_what"')
        self.assertInResponse(u'OK')


class MusicDatabaseListTest(protocol.BaseTestCase):
    def test_list_foo_returns_ack(self):
        self.sendRequest(u'list "foo"')
        self.assertEqualResponse(u'ACK [2@0] {list} incorrect arguments')

    ### Artist

    def test_list_artist_with_quotes(self):
        self.sendRequest(u'list "artist"')
        self.assertInResponse(u'OK')

    def test_list_artist_without_quotes(self):
        self.sendRequest(u'list artist')
        self.assertInResponse(u'OK')

    def test_list_artist_without_quotes_and_capitalized(self):
        self.sendRequest(u'list Artist')
        self.assertInResponse(u'OK')

    def test_list_artist_with_query_of_one_token(self):
        self.sendRequest(u'list "artist" "anartist"')
        self.assertEqualResponse(
            u'ACK [2@0] {list} should be "Album" for 3 arguments')

    def test_list_artist_with_unknown_field_in_query_returns_ack(self):
        self.sendRequest(u'list "artist" "foo" "bar"')
        self.assertEqualResponse(u'ACK [2@0] {list} not able to parse args')

    def test_list_artist_by_artist(self):
        self.sendRequest(u'list "artist" "artist" "anartist"')
        self.assertInResponse(u'OK')

    def test_list_artist_by_album(self):
        self.sendRequest(u'list "artist" "album" "analbum"')
        self.assertInResponse(u'OK')

    def test_list_artist_by_full_date(self):
        self.sendRequest(u'list "artist" "date" "2001-01-01"')
        self.assertInResponse(u'OK')

    def test_list_artist_by_year(self):
        self.sendRequest(u'list "artist" "date" "2001"')
        self.assertInResponse(u'OK')

    def test_list_artist_by_genre(self):
        self.sendRequest(u'list "artist" "genre" "agenre"')
        self.assertInResponse(u'OK')

    def test_list_artist_by_artist_and_album(self):
        self.sendRequest(
            u'list "artist" "artist" "anartist" "album" "analbum"')
        self.assertInResponse(u'OK')

    ### Album

    def test_list_album_with_quotes(self):
        self.sendRequest(u'list "album"')
        self.assertInResponse(u'OK')

    def test_list_album_without_quotes(self):
        self.sendRequest(u'list album')
        self.assertInResponse(u'OK')

    def test_list_album_without_quotes_and_capitalized(self):
        self.sendRequest(u'list Album')
        self.assertInResponse(u'OK')

    def test_list_album_with_artist_name(self):
        self.sendRequest(u'list "album" "anartist"')
        self.assertInResponse(u'OK')

    def test_list_album_by_artist(self):
        self.sendRequest(u'list "album" "artist" "anartist"')
        self.assertInResponse(u'OK')

    def test_list_album_by_album(self):
        self.sendRequest(u'list "album" "album" "analbum"')
        self.assertInResponse(u'OK')

    def test_list_album_by_full_date(self):
        self.sendRequest(u'list "album" "date" "2001-01-01"')
        self.assertInResponse(u'OK')

    def test_list_album_by_year(self):
        self.sendRequest(u'list "album" "date" "2001"')
        self.assertInResponse(u'OK')

    def test_list_album_by_genre(self):
        self.sendRequest(u'list "album" "genre" "agenre"')
        self.assertInResponse(u'OK')

    def test_list_album_by_artist_and_album(self):
        self.sendRequest(
            u'list "album" "artist" "anartist" "album" "analbum"')
        self.assertInResponse(u'OK')

    ### Date

    def test_list_date_with_quotes(self):
        self.sendRequest(u'list "date"')
        self.assertInResponse(u'OK')

    def test_list_date_without_quotes(self):
        self.sendRequest(u'list date')
        self.assertInResponse(u'OK')

    def test_list_date_without_quotes_and_capitalized(self):
        self.sendRequest(u'list Date')
        self.assertInResponse(u'OK')

    def test_list_date_with_query_of_one_token(self):
        self.sendRequest(u'list "date" "anartist"')
        self.assertEqualResponse(
            u'ACK [2@0] {list} should be "Album" for 3 arguments')

    def test_list_date_by_artist(self):
        self.sendRequest(u'list "date" "artist" "anartist"')
        self.assertInResponse(u'OK')

    def test_list_date_by_album(self):
        self.sendRequest(u'list "date" "album" "analbum"')
        self.assertInResponse(u'OK')

    def test_list_date_by_full_date(self):
        self.sendRequest(u'list "date" "date" "2001-01-01"')
        self.assertInResponse(u'OK')

    def test_list_date_by_year(self):
        self.sendRequest(u'list "date" "date" "2001"')
        self.assertInResponse(u'OK')

    def test_list_date_by_genre(self):
        self.sendRequest(u'list "date" "genre" "agenre"')
        self.assertInResponse(u'OK')

    def test_list_date_by_artist_and_album(self):
        self.sendRequest(u'list "date" "artist" "anartist" "album" "analbum"')
        self.assertInResponse(u'OK')

    ### Genre

    def test_list_genre_with_quotes(self):
        self.sendRequest(u'list "genre"')
        self.assertInResponse(u'OK')

    def test_list_genre_without_quotes(self):
        self.sendRequest(u'list genre')
        self.assertInResponse(u'OK')

    def test_list_genre_without_quotes_and_capitalized(self):
        self.sendRequest(u'list Genre')
        self.assertInResponse(u'OK')

    def test_list_genre_with_query_of_one_token(self):
        self.sendRequest(u'list "genre" "anartist"')
        self.assertEqualResponse(
            u'ACK [2@0] {list} should be "Album" for 3 arguments')

    def test_list_genre_by_artist(self):
        self.sendRequest(u'list "genre" "artist" "anartist"')
        self.assertInResponse(u'OK')

    def test_list_genre_by_album(self):
        self.sendRequest(u'list "genre" "album" "analbum"')
        self.assertInResponse(u'OK')

    def test_list_genre_by_full_date(self):
        self.sendRequest(u'list "genre" "date" "2001-01-01"')
        self.assertInResponse(u'OK')

    def test_list_genre_by_year(self):
        self.sendRequest(u'list "genre" "date" "2001"')
        self.assertInResponse(u'OK')

    def test_list_genre_by_genre(self):
        self.sendRequest(u'list "genre" "genre" "agenre"')
        self.assertInResponse(u'OK')

    def test_list_genre_by_artist_and_album(self):
        self.sendRequest(
            u'list "genre" "artist" "anartist" "album" "analbum"')
        self.assertInResponse(u'OK')


class MusicDatabaseSearchTest(protocol.BaseTestCase):
    def test_search_album(self):
        self.sendRequest(u'search "album" "analbum"')
        self.assertInResponse(u'OK')

    def test_search_album_without_quotes(self):
        self.sendRequest(u'search album "analbum"')
        self.assertInResponse(u'OK')

    def test_search_artist(self):
        self.sendRequest(u'search "artist" "anartist"')
        self.assertInResponse(u'OK')

    def test_search_artist_without_quotes(self):
        self.sendRequest(u'search artist "anartist"')
        self.assertInResponse(u'OK')

    def test_search_filename(self):
        self.sendRequest(u'search "filename" "afilename"')
        self.assertInResponse(u'OK')

    def test_search_filename_without_quotes(self):
        self.sendRequest(u'search filename "afilename"')
        self.assertInResponse(u'OK')

    def test_search_title(self):
        self.sendRequest(u'search "title" "atitle"')
        self.assertInResponse(u'OK')

    def test_search_title_without_quotes(self):
        self.sendRequest(u'search title "atitle"')
        self.assertInResponse(u'OK')

    def test_search_any(self):
        self.sendRequest(u'search "any" "anything"')
        self.assertInResponse(u'OK')

    def test_search_any_without_quotes(self):
        self.sendRequest(u'search any "anything"')
        self.assertInResponse(u'OK')

    def test_search_date(self):
        self.sendRequest(u'search "date" "2002-01-01"')
        self.assertInResponse(u'OK')

    def test_search_date_without_quotes(self):
        self.sendRequest(u'search date "2002-01-01"')
        self.assertInResponse(u'OK')

    def test_search_date_with_capital_d_and_incomplete_date(self):
        self.sendRequest(u'search Date "2005"')
        self.assertInResponse(u'OK')

    def test_search_else_should_fail(self):
        self.sendRequest(u'search "sometype" "something"')
        self.assertEqualResponse(u'ACK [2@0] {search} incorrect arguments')



import os

from mopidy import settings as default_settings_module, SettingsError
from mopidy.utils.settings import (format_settings_list, mask_value_if_secret,
    SettingsProxy, validate_settings)

from tests import unittest


class ValidateSettingsTest(unittest.TestCase):
    def setUp(self):
        self.defaults = {
            'MPD_SERVER_HOSTNAME': '::',
            'MPD_SERVER_PORT': 6600,
            'SPOTIFY_BITRATE': 160,
        }

    def test_no_errors_yields_empty_dict(self):
        result = validate_settings(self.defaults, {})
        self.assertEqual(result, {})

    def test_unknown_setting_returns_error(self):
        result = validate_settings(self.defaults,
            {'MPD_SERVER_HOSTNMAE': '127.0.0.1'})
        self.assertEqual(result['MPD_SERVER_HOSTNMAE'],
            u'Unknown setting. Is it misspelled?')

    def test_not_renamed_setting_returns_error(self):
        result = validate_settings(self.defaults,
            {'SERVER_HOSTNAME': '127.0.0.1'})
        self.assertEqual(result['SERVER_HOSTNAME'],
            u'Deprecated setting. Use MPD_SERVER_HOSTNAME.')

    def test_unneeded_settings_returns_error(self):
        result = validate_settings(self.defaults,
            {'SPOTIFY_LIB_APPKEY': '/tmp/foo'})
        self.assertEqual(result['SPOTIFY_LIB_APPKEY'],
            u'Deprecated setting. It may be removed.')

    def test_deprecated_setting_value_returns_error(self):
        result = validate_settings(self.defaults,
            {'BACKENDS': ('mopidy.backends.despotify.DespotifyBackend',)})
        self.assertEqual(result['BACKENDS'],
            u'Deprecated setting value. ' +
            '"mopidy.backends.despotify.DespotifyBackend" is no longer ' +
            'available.')

    def test_unavailable_bitrate_setting_returns_error(self):
        result = validate_settings(self.defaults,
            {'SPOTIFY_BITRATE': 50})
        self.assertEqual(result['SPOTIFY_BITRATE'],
            u'Unavailable Spotify bitrate. ' +
            u'Available bitrates are 96, 160, and 320.')

    def test_two_errors_are_both_reported(self):
        result = validate_settings(self.defaults,
            {'FOO': '', 'BAR': ''})
        self.assertEqual(len(result), 2)

    def test_masks_value_if_secret(self):
        secret = mask_value_if_secret('SPOTIFY_PASSWORD', 'bar')
        self.assertEqual(u'********', secret)

    def test_does_not_mask_value_if_not_secret(self):
        not_secret = mask_value_if_secret('SPOTIFY_USERNAME', 'foo')
        self.assertEqual('foo', not_secret)

    def test_does_not_mask_value_if_none(self):
        not_secret = mask_value_if_secret('SPOTIFY_USERNAME', None)
        self.assertEqual(None, not_secret)


class SettingsProxyTest(unittest.TestCase):
    def setUp(self):
        self.settings = SettingsProxy(default_settings_module)
        self.settings.local.clear()

    def test_set_and_get_attr(self):
        self.settings.TEST = 'test'
        self.assertEqual(self.settings.TEST, 'test')

    def test_getattr_raises_error_on_missing_setting(self):
        try:
            _ = self.settings.TEST
            self.fail(u'Should raise exception')
        except SettingsError as e:
            self.assertEqual(u'Setting "TEST" is not set.', e.message)

    def test_getattr_raises_error_on_empty_setting(self):
        self.settings.TEST = u''
        try:
            _ = self.settings.TEST
            self.fail(u'Should raise exception')
        except SettingsError as e:
            self.assertEqual(u'Setting "TEST" is empty.', e.message)

    def test_getattr_does_not_raise_error_if_setting_is_false(self):
        self.settings.TEST = False
        self.assertEqual(False, self.settings.TEST)

    def test_getattr_does_not_raise_error_if_setting_is_none(self):
        self.settings.TEST = None
        self.assertEqual(None, self.settings.TEST)

    def test_getattr_does_not_raise_error_if_setting_is_zero(self):
        self.settings.TEST = 0
        self.assertEqual(0, self.settings.TEST)

    def test_setattr_updates_runtime_settings(self):
        self.settings.TEST = 'test'
        self.assert_('TEST' in self.settings.runtime)

    def test_setattr_updates_runtime_with_value(self):
        self.settings.TEST = 'test'
        self.assertEqual(self.settings.runtime['TEST'], 'test')

    def test_runtime_value_included_in_current(self):
        self.settings.TEST = 'test'
        self.assertEqual(self.settings.current['TEST'], 'test')

    def test_value_ending_in_path_is_expanded(self):
        self.settings.TEST_PATH = '~/test'
        actual = self.settings.TEST_PATH
        expected = os.path.expanduser('~/test')
        self.assertEqual(actual, expected)

    def test_value_ending_in_path_is_absolute(self):
        self.settings.TEST_PATH = './test'
        actual = self.settings.TEST_PATH
        expected = os.path.abspath('./test')
        self.assertEqual(actual, expected)

    def test_value_ending_in_file_is_expanded(self):
        self.settings.TEST_FILE = '~/test'
        actual = self.settings.TEST_FILE
        expected = os.path.expanduser('~/test')
        self.assertEqual(actual, expected)

    def test_value_ending_in_file_is_absolute(self):
        self.settings.TEST_FILE = './test'
        actual = self.settings.TEST_FILE
        expected = os.path.abspath('./test')
        self.assertEqual(actual, expected)

    def test_value_not_ending_in_path_or_file_is_not_expanded(self):
        self.settings.TEST = '~/test'
        actual = self.settings.TEST
        self.assertEqual(actual, '~/test')

    def test_value_not_ending_in_path_or_file_is_not_absolute(self):
        self.settings.TEST = './test'
        actual = self.settings.TEST
        self.assertEqual(actual, './test')

    def test_value_ending_in_file_can_be_none(self):
        self.settings.TEST_FILE = None
        self.assertEqual(self.settings.TEST_FILE, None)

    def test_value_ending_in_path_can_be_none(self):
        self.settings.TEST_PATH = None
        self.assertEqual(self.settings.TEST_PATH, None)

    def test_interactive_input_of_missing_defaults(self):
        self.settings.default['TEST'] = ''
        interactive_input = 'input'
        self.settings._read_from_stdin = lambda _: interactive_input
        self.settings.validate(interactive=True)
        self.assertEqual(interactive_input, self.settings.TEST)

    def test_interactive_input_not_needed_when_setting_is_set_locally(self):
        self.settings.default['TEST'] = ''
        self.settings.local['TEST'] = 'test'
        self.settings._read_from_stdin = lambda _: self.fail(
            'Should not read from stdin')
        self.settings.validate(interactive=True)


class FormatSettingListTest(unittest.TestCase):
    def setUp(self):
        self.settings = SettingsProxy(default_settings_module)

    def test_contains_the_setting_name(self):
        self.settings.TEST = u'test'
        result = format_settings_list(self.settings)
        self.assert_('TEST:' in result, result)

    def test_repr_of_a_string_value(self):
        self.settings.TEST = u'test'
        result = format_settings_list(self.settings)
        self.assert_("TEST: u'test'" in result, result)

    def test_repr_of_an_int_value(self):
        self.settings.TEST = 123
        result = format_settings_list(self.settings)
        self.assert_("TEST: 123" in result, result)

    def test_repr_of_a_tuple_value(self):
        self.settings.TEST = (123, u'abc')
        result = format_settings_list(self.settings)
        self.assert_("TEST: (123, u'abc')" in result, result)

    def test_passwords_are_masked(self):
        self.settings.TEST_PASSWORD = u'secret'
        result = format_settings_list(self.settings)
        self.assert_("TEST_PASSWORD: u'secret'" not in result, result)
        self.assert_("TEST_PASSWORD: u'********'" in result, result)

    def test_short_values_are_not_pretty_printed(self):
        self.settings.FRONTEND = (u'mopidy.frontends.mpd.MpdFrontend',)
        result = format_settings_list(self.settings)
        self.assert_("FRONTEND: (u'mopidy.frontends.mpd.MpdFrontend',)" in result,
            result)

    def test_long_values_are_pretty_printed(self):
        self.settings.FRONTEND = (u'mopidy.frontends.mpd.MpdFrontend',
            u'mopidy.frontends.lastfm.LastfmFrontend')
        result = format_settings_list(self.settings)
        self.assert_("""FRONTEND: 
  (u'mopidy.frontends.mpd.MpdFrontend',
   u'mopidy.frontends.lastfm.LastfmFrontend')""" in result, result)



from mopidy.outputs import BaseOutput

class LocalOutput(BaseOutput):
    """
    Basic output to local audio sink.

    This output will normally tell GStreamer to choose whatever it thinks is
    best for your system. In other words this is usually a sane choice.

    **Dependencies:**

    - None

    **Settings:**

    - None
    """

    def describe_bin(self):
        return 'autoaudiosink'



import logging
import os
import sys
import re
import urllib

logger = logging.getLogger('mopidy.utils.path')

def get_or_create_folder(folder):
    folder = os.path.expanduser(folder)
    if os.path.isfile(folder):
        raise OSError('A file with the same name as the desired ' \
            'dir, "%s", already exists.' % folder)
    elif not os.path.isdir(folder):
        logger.info(u'Creating dir %s', folder)
        os.makedirs(folder, 0755)
    return folder

def get_or_create_file(filename):
    filename = os.path.expanduser(filename)
    if not os.path.isfile(filename):
        logger.info(u'Creating file %s', filename)
        open(filename, 'w')
    return filename

def path_to_uri(*paths):
    path = os.path.join(*paths)
    path = path.encode('utf-8')
    if sys.platform == 'win32':
        return 'file:' + urllib.pathname2url(path)
    return 'file://' + urllib.pathname2url(path)

def uri_to_path(uri):
    if sys.platform == 'win32':
        path = urllib.url2pathname(re.sub('^file:', '', uri))
    else:
        path = urllib.url2pathname(re.sub('^file://', '', uri))
    return path.encode('latin1').decode('utf-8') # Undo double encoding

def split_path(path):
    parts = []
    while True:
        path, part = os.path.split(path)
        if part:
            parts.insert(0, part)
        if not path or path == '/':
            break
    return parts

# pylint: disable = W0612
# Unused variable 'dirnames'
def find_files(path):
    if os.path.isfile(path):
        if not isinstance(path, unicode):
            path = path.decode('utf-8')
        yield path
    else:
        for dirpath, dirnames, filenames in os.walk(path):
            for filename in filenames:
                filename = os.path.join(dirpath, filename)
                if not isinstance(filename, unicode):
                    filename = filename.decode('utf-8')
                yield filename
# pylint: enable = W0612

# FIXME replace with mock usage in tests.
class Mtime(object):
    def __init__(self):
        self.fake = None

    def __call__(self, path):
        if self.fake is not None:
            return self.fake
        return int(os.stat(path).st_mtime)

    def set_fake_time(self, time):
        self.fake = time

    def undo_fake(self):
        self.fake = None

mtime = Mtime()



import logging
import re

from pykka import ActorDeadError
from pykka.registry import ActorRegistry

from mopidy import settings
from mopidy.backends.base import Backend
from mopidy.frontends.mpd import exceptions
from mopidy.frontends.mpd.protocol import mpd_commands, request_handlers
# Do not remove the following import. The protocol modules must be imported to
# get them registered as request handlers.
# pylint: disable = W0611
from mopidy.frontends.mpd.protocol import (audio_output, command_list,
    connection, current_playlist, empty, music_db, playback, reflection,
    status, stickers, stored_playlists)
# pylint: enable = W0611
from mopidy.mixers.base import BaseMixer
from mopidy.utils import flatten

logger = logging.getLogger('mopidy.frontends.mpd.dispatcher')

class MpdDispatcher(object):
    """
    The MPD session feeds the MPD dispatcher with requests. The dispatcher
    finds the correct handler, processes the request and sends the response
    back to the MPD session.
    """

    _noidle = re.compile(r'^noidle$')

    def __init__(self, session=None):
        self.authenticated = False
        self.command_list = False
        self.command_list_ok = False
        self.command_list_index = None
        self.context = MpdContext(self, session=session)

    def handle_request(self, request, current_command_list_index=None):
        """Dispatch incoming requests to the correct handler."""
        self.command_list_index = current_command_list_index
        response = []
        filter_chain = [
            self._catch_mpd_ack_errors_filter,
            self._authenticate_filter,
            self._command_list_filter,
            self._idle_filter,
            self._add_ok_filter,
            self._call_handler_filter,
        ]
        return self._call_next_filter(request, response, filter_chain)

    def handle_idle(self, subsystem):
        self.context.events.add(subsystem)

        subsystems = self.context.subscriptions.intersection(
            self.context.events)
        if not subsystems:
            return

        response = []
        for subsystem in subsystems:
            response.append(u'changed: %s' % subsystem)
        response.append(u'OK')
        self.context.subscriptions = set()
        self.context.events = set()
        self.context.session.send_lines(response)

    def _call_next_filter(self, request, response, filter_chain):
        if filter_chain:
            next_filter = filter_chain.pop(0)
            return next_filter(request, response, filter_chain)
        else:
            return response


    ### Filter: catch MPD ACK errors

    def _catch_mpd_ack_errors_filter(self, request, response, filter_chain):
        try:
            return self._call_next_filter(request, response, filter_chain)
        except exceptions.MpdAckError as mpd_ack_error:
            if self.command_list_index is not None:
                mpd_ack_error.index = self.command_list_index
            return [mpd_ack_error.get_mpd_ack()]


    ### Filter: authenticate

    def _authenticate_filter(self, request, response, filter_chain):
        if self.authenticated:
            return self._call_next_filter(request, response, filter_chain)
        elif settings.MPD_SERVER_PASSWORD is None:
            self.authenticated = True
            return self._call_next_filter(request, response, filter_chain)
        else:
            command_name = request.split(' ')[0]
            command_names_not_requiring_auth = [
                command.name for command in mpd_commands
                if not command.auth_required]
            if command_name in command_names_not_requiring_auth:
                return self._call_next_filter(request, response, filter_chain)
            else:
                raise exceptions.MpdPermissionError(command=command_name)


    ### Filter: command list

    def _command_list_filter(self, request, response, filter_chain):
        if self._is_receiving_command_list(request):
            self.command_list.append(request)
            return []
        else:
            response = self._call_next_filter(request, response, filter_chain)
            if (self._is_receiving_command_list(request) or
                    self._is_processing_command_list(request)):
                if response and response[-1] == u'OK':
                    response = response[:-1]
            return response

    def _is_receiving_command_list(self, request):
        return (self.command_list is not False
            and request != u'command_list_end')

    def _is_processing_command_list(self, request):
        return (self.command_list_index is not None
            and request != u'command_list_end')


    ### Filter: idle

    def _idle_filter(self, request, response, filter_chain):
        if self._is_currently_idle() and not self._noidle.match(request):
            logger.debug(u'Client sent us %s, only %s is allowed while in '
                'the idle state', repr(request), repr(u'noidle'))
            self.context.session.close()
            return []

        if not self._is_currently_idle() and self._noidle.match(request):
            return [] # noidle was called before idle

        response = self._call_next_filter(request, response, filter_chain)

        if self._is_currently_idle():
            return []
        else:
            return response

    def _is_currently_idle(self):
        return bool(self.context.subscriptions)


    ### Filter: add OK

    def _add_ok_filter(self, request, response, filter_chain):
        response = self._call_next_filter(request, response, filter_chain)
        if not self._has_error(response):
            response.append(u'OK')
        return response

    def _has_error(self, response):
        return response and response[-1].startswith(u'ACK')


    ### Filter: call handler

    def _call_handler_filter(self, request, response, filter_chain):
        try:
            response = self._format_response(self._call_handler(request))
            return self._call_next_filter(request, response, filter_chain)
        except ActorDeadError as e:
            logger.warning(u'Tried to communicate with dead actor.')
            raise exceptions.MpdSystemError(e)

    def _call_handler(self, request):
        (handler, kwargs) = self._find_handler(request)
        return handler(self.context, **kwargs)

    def _find_handler(self, request):
        for pattern in request_handlers:
            matches = re.match(pattern, request)
            if matches is not None:
                return (request_handlers[pattern], matches.groupdict())
        command_name = request.split(' ')[0]
        if command_name in [command.name for command in mpd_commands]:
            raise exceptions.MpdArgError(u'incorrect arguments',
                command=command_name)
        raise exceptions.MpdUnknownCommand(command=command_name)

    def _format_response(self, response):
        formatted_response = []
        for element in self._listify_result(response):
            formatted_response.extend(self._format_lines(element))
        return formatted_response

    def _listify_result(self, result):
        if result is None:
            return []
        if isinstance(result, set):
            return flatten(list(result))
        if not isinstance(result, list):
            return [result]
        return flatten(result)

    def _format_lines(self, line):
        if isinstance(line, dict):
            return [u'%s: %s' % (key, value) for (key, value) in line.items()]
        if isinstance(line, tuple):
            (key, value) = line
            return [u'%s: %s' % (key, value)]
        return [line]


class MpdContext(object):
    """
    This object is passed as the first argument to all MPD command handlers to
    give the command handlers access to important parts of Mopidy.
    """

    #: The current :class:`MpdDispatcher`.
    dispatcher = None

    #: The current :class:`mopidy.frontends.mpd.MpdSession`.
    session = None

    #: The active subsystems that have pending events.
    events = None

    #: The subsytems that we want to be notified about in idle mode.
    subscriptions = None

    def __init__(self, dispatcher, session=None):
        self.dispatcher = dispatcher
        self.session = session
        self.events = set()
        self.subscriptions = set()
        self._backend = None
        self._mixer = None

    @property
    def backend(self):
        """
        The backend. An instance of :class:`mopidy.backends.base.Backend`.
        """
        if self._backend is None:
            backend_refs = ActorRegistry.get_by_class(Backend)
            assert len(backend_refs) == 1, \
                'Expected exactly one running backend.'
            self._backend = backend_refs[0].proxy()
        return self._backend

    @property
    def mixer(self):
        """
        The mixer. An instance of :class:`mopidy.mixers.base.BaseMixer`.
        """
        if self._mixer is None:
            mixer_refs = ActorRegistry.get_by_class(BaseMixer)
            assert len(mixer_refs) == 1, 'Expected exactly one running mixer.'
            self._mixer = mixer_refs[0].proxy()
        return self._mixer



from tests.frontends.mpd import protocol


class StickersHandlerTest(protocol.BaseTestCase):
    def test_sticker_get(self):
        self.sendRequest(
            u'sticker get "song" "file:///dev/urandom" "a_name"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_sticker_set(self):
        self.sendRequest(
            u'sticker set "song" "file:///dev/urandom" "a_name" "a_value"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_sticker_delete_with_name(self):
        self.sendRequest(
            u'sticker delete "song" "file:///dev/urandom" "a_name"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_sticker_delete_without_name(self):
        self.sendRequest(
            u'sticker delete "song" "file:///dev/urandom"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_sticker_list(self):
        self.sendRequest(
            u'sticker list "song" "file:///dev/urandom"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')

    def test_sticker_find(self):
        self.sendRequest(
            u'sticker find "song" "file:///dev/urandom" "a_name"')
        self.assertEqualResponse(u'ACK [0@0] {} Not implemented')



from pykka import registry

class BackendListener(object):
    """
    Marker interface for recipients of events sent by the backend.

    Any Pykka actor that mixes in this class will receive calls to the methods
    defined here when the corresponding events happen in the backend. This
    interface is used both for looking up what actors to notify of the events,
    and for providing default implementations for those listeners that are not
    interested in all events.
    """

    @staticmethod
    def send(event, **kwargs):
        """Helper to allow calling of backend listener events"""
        # FIXME this should be updated once Pykka supports non-blocking calls
        # on proxies or some similar solution.
        registry.ActorRegistry.broadcast({
            'command': 'pykka_call',
            'attr_path': (event,),
            'args': [],
            'kwargs': kwargs,
        }, target_class=BackendListener)

    def track_playback_paused(self, track, time_position):
        """
        Called whenever track playback is paused.

        *MAY* be implemented by actor.

        :param track: the track that was playing when playback paused
        :type track: :class:`mopidy.models.Track`
        :param time_position: the time position in milliseconds
        :type time_position: int
        """
        pass

    def track_playback_resumed(self, track, time_position):
        """
        Called whenever track playback is resumed.

        *MAY* be implemented by actor.

        :param track: the track that was playing when playback resumed
        :type track: :class:`mopidy.models.Track`
        :param time_position: the time position in milliseconds
        :type time_position: int
        """
        pass


    def track_playback_started(self, track):
        """
        Called whenever a new track starts playing.

        *MAY* be implemented by actor.

        :param track: the track that just started playing
        :type track: :class:`mopidy.models.Track`
        """
        pass

    def track_playback_ended(self, track, time_position):
        """
        Called whenever playback of a track ends.

        *MAY* be implemented by actor.

        :param track: the track that was played before playback stopped
        :type track: :class:`mopidy.models.Track`
        :param time_position: the time position in milliseconds
        :type time_position: int
        """
        pass

    def playback_state_changed(self):
        """
        Called whenever playback state is changed.

        *MAY* be implemented by actor.
        """
        pass

    def playlist_changed(self):
        """
        Called whenever a playlist is changed.

        *MAY* be implemented by actor.
        """
        pass

    def options_changed(self):
        """
        Called whenever an option is changed.

        *MAY* be implemented by actor.
        """
        pass

    def volume_changed(self):
        """
        Called whenever the volume is changed.

        *MAY* be implemented by actor.
        """
        pass

    def seeked(self):
        """
        Called whenever the time position changes by an unexpected amount, e.g.
        at seek to a new time position.

        *MAY* be implemented by actor.
        """
        pass



from copy import copy
import logging
import random

from mopidy.listeners import BackendListener
from mopidy.models import CpTrack

logger = logging.getLogger('mopidy.backends.base')

class CurrentPlaylistController(object):
    """
    :param backend: backend the controller is a part of
    :type backend: :class:`mopidy.backends.base.Backend`
    """

    pykka_traversable = True

    def __init__(self, backend):
        self.backend = backend
        self.cp_id = 0
        self._cp_tracks = []
        self._version = 0

    @property
    def cp_tracks(self):
        """
        List of two-tuples of (CPID integer, :class:`mopidy.models.Track`).

        Read-only.
        """
        return [copy(cp_track) for cp_track in self._cp_tracks]

    @property
    def tracks(self):
        """
        List of :class:`mopidy.models.Track` in the current playlist.

        Read-only.
        """
        return [cp_track.track for cp_track in self._cp_tracks]

    @property
    def length(self):
        """
        Length of the current playlist.
        """
        return len(self._cp_tracks)

    @property
    def version(self):
        """
        The current playlist version. Integer which is increased every time the
        current playlist is changed. Is not reset before Mopidy is restarted.
        """
        return self._version

    @version.setter
    def version(self, version):
        self._version = version
        self.backend.playback.on_current_playlist_change()
        self._trigger_playlist_changed()

    def add(self, track, at_position=None, increase_version=True):
        """
        Add the track to the end of, or at the given position in the current
        playlist.

        :param track: track to add
        :type track: :class:`mopidy.models.Track`
        :param at_position: position in current playlist to add track
        :type at_position: int or :class:`None`
        :rtype: two-tuple of (CPID integer, :class:`mopidy.models.Track`) that
            was added to the current playlist playlist
        """
        assert at_position <= len(self._cp_tracks), \
            u'at_position can not be greater than playlist length'
        cp_track = CpTrack(self.cp_id, track)
        if at_position is not None:
            self._cp_tracks.insert(at_position, cp_track)
        else:
            self._cp_tracks.append(cp_track)
        if increase_version:
            self.version += 1
        self.cp_id += 1
        return cp_track

    def append(self, tracks):
        """
        Append the given tracks to the current playlist.

        :param tracks: tracks to append
        :type tracks: list of :class:`mopidy.models.Track`
        """
        for track in tracks:
            self.add(track, increase_version=False)

        if tracks:
            self.version += 1

    def clear(self):
        """Clear the current playlist."""
        self._cp_tracks = []
        self.version += 1

    def get(self, **criteria):
        """
        Get track by given criterias from current playlist.

        Raises :exc:`LookupError` if a unique match is not found.

        Examples::

            get(cpid=7)             # Returns track with CPID 7
                                    # (current playlist ID)
            get(id=1)               # Returns track with ID 1
            get(uri='xyz')          # Returns track with URI 'xyz'
            get(id=1, uri='xyz')    # Returns track with ID 1 and URI 'xyz'

        :param criteria: on or more criteria to match by
        :type criteria: dict
        :rtype: two-tuple (CPID integer, :class:`mopidy.models.Track`)
        """
        matches = self._cp_tracks
        for (key, value) in criteria.iteritems():
            if key == 'cpid':
                matches = filter(lambda ct: ct.cpid == value, matches)
            else:
                matches = filter(lambda ct: getattr(ct.track, key) == value,
                    matches)
        if len(matches) == 1:
            return matches[0]
        criteria_string = ', '.join(
            ['%s=%s' % (k, v) for (k, v) in criteria.iteritems()])
        if len(matches) == 0:
            raise LookupError(u'"%s" match no tracks' % criteria_string)
        else:
            raise LookupError(u'"%s" match multiple tracks' % criteria_string)

    def index(self, cp_track):
        """
        Get index of the given (CPID integer, :class:`mopidy.models.Track`)
        two-tuple in the current playlist.

        Raises :exc:`ValueError` if not found.

        :param cp_track: track to find the index of
        :type cp_track: two-tuple (CPID integer, :class:`mopidy.models.Track`)
        :rtype: int
        """
        return self._cp_tracks.index(cp_track)

    def move(self, start, end, to_position):
        """
        Move the tracks in the slice ``[start:end]`` to ``to_position``.

        :param start: position of first track to move
        :type start: int
        :param end: position after last track to move
        :type end: int
        :param to_position: new position for the tracks
        :type to_position: int
        """
        if start == end:
            end += 1

        cp_tracks = self._cp_tracks

        assert start < end, 'start must be smaller than end'
        assert start >= 0, 'start must be at least zero'
        assert end <= len(cp_tracks), \
            'end can not be larger than playlist length'
        assert to_position >= 0, 'to_position must be at least zero'
        assert to_position <= len(cp_tracks), \
            'to_position can not be larger than playlist length'

        new_cp_tracks = cp_tracks[:start] + cp_tracks[end:]
        for cp_track in cp_tracks[start:end]:
            new_cp_tracks.insert(to_position, cp_track)
            to_position += 1
        self._cp_tracks = new_cp_tracks
        self.version += 1

    def remove(self, **criteria):
        """
        Remove the track from the current playlist.

        Uses :meth:`get()` to lookup the track to remove.

        :param criteria: on or more criteria to match by
        :type criteria: dict
        """
        cp_track = self.get(**criteria)
        position = self._cp_tracks.index(cp_track)
        del self._cp_tracks[position]
        self.version += 1

    def shuffle(self, start=None, end=None):
        """
        Shuffles the entire playlist. If ``start`` and ``end`` is given only
        shuffles the slice ``[start:end]``.

        :param start: position of first track to shuffle
        :type start: int or :class:`None`
        :param end: position after last track to shuffle
        :type end: int or :class:`None`
        """
        cp_tracks = self._cp_tracks

        if start is not None and end is not None:
            assert start < end, 'start must be smaller than end'

        if start is not None:
            assert start >= 0, 'start must be at least zero'

        if end is not None:
            assert end <= len(cp_tracks), 'end can not be larger than ' + \
                'playlist length'

        before = cp_tracks[:start or 0]
        shuffled = cp_tracks[start:end]
        after = cp_tracks[end or len(cp_tracks):]
        random.shuffle(shuffled)
        self._cp_tracks = before + shuffled + after
        self.version += 1

    def slice(self, start, end):
        """
        Returns a slice of the current playlist, limited by the given
        start and end positions.

        :param start: position of first track to include in slice
        :type start: int
        :param end: position after last track to include in slice
        :type end: int
        :rtype: two-tuple of (CPID integer, :class:`mopidy.models.Track`)
        """
        return [copy(cp_track) for cp_track in self._cp_tracks[start:end]]

    def _trigger_playlist_changed(self):
        logger.debug(u'Triggering playlist changed event')
        BackendListener.send('playlist_changed')
